"""Raw representations of every data type in the AWS Logs service.

See Also:
    `AWS developer guide for Logs
    <https://docs.aws.amazon.com/AmazonCloudWatch/latest/logs/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Destination",
    "DestinationProperties",
    "LogGroup",
    "LogGroupProperties",
    "LogStream",
    "LogStreamProperties",
    "MetricFilter",
    "MetricFilterProperties",
    "SubscriptionFilter",
    "SubscriptionFilterProperties",
]


@attrs(**ATTRSCONFIG)
class DestinationProperties(ResourceProperties):
    DestinationName = attrib(default=None)
    DestinationPolicy = attrib(default=None)
    RoleArn = attrib(default=None)
    TargetArn = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Destination(Resource):
    """A Destination for Logs.

    See Also:
        `AWS Cloud Formation documentation for Destination
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-destination.html>`_
    """

    RESOURCE_TYPE = "AWS::Logs::Destination"

    Properties: DestinationProperties = attrib(
        factory=DestinationProperties,
        converter=create_object_converter(DestinationProperties),
    )


@attrs(**ATTRSCONFIG)
class LogGroupProperties(ResourceProperties):
    LogGroupName = attrib(default=None)
    RetentionInDays = attrib(default=None)


@attrs(**ATTRSCONFIG)
class LogGroup(Resource):
    """A Log Group for Logs.

    See Also:
        `AWS Cloud Formation documentation for LogGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-loggroup.html>`_
    """

    RESOURCE_TYPE = "AWS::Logs::LogGroup"

    Properties: LogGroupProperties = attrib(
        factory=LogGroupProperties,
        converter=create_object_converter(LogGroupProperties),
    )


@attrs(**ATTRSCONFIG)
class LogStreamProperties(ResourceProperties):
    LogGroupName = attrib(default=None)
    LogStreamName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class LogStream(Resource):
    """A Log Stream for Logs.

    See Also:
        `AWS Cloud Formation documentation for LogStream
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-logstream.html>`_
    """

    RESOURCE_TYPE = "AWS::Logs::LogStream"

    Properties: LogStreamProperties = attrib(
        factory=LogStreamProperties,
        converter=create_object_converter(LogStreamProperties),
    )


@attrs(**ATTRSCONFIG)
class MetricFilterProperties(ResourceProperties):
    FilterPattern = attrib(default=None)
    LogGroupName = attrib(default=None)
    MetricTransformations = attrib(default=None)


@attrs(**ATTRSCONFIG)
class MetricFilter(Resource):
    """A Metric Filter for Logs.

    See Also:
        `AWS Cloud Formation documentation for MetricFilter
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-metricfilter.html>`_
    """

    RESOURCE_TYPE = "AWS::Logs::MetricFilter"

    Properties: MetricFilterProperties = attrib(
        factory=MetricFilterProperties,
        converter=create_object_converter(MetricFilterProperties),
    )


@attrs(**ATTRSCONFIG)
class SubscriptionFilterProperties(ResourceProperties):
    DestinationArn = attrib(default=None)
    FilterPattern = attrib(default=None)
    LogGroupName = attrib(default=None)
    RoleArn = attrib(default=None)


@attrs(**ATTRSCONFIG)
class SubscriptionFilter(Resource):
    """A Subscription Filter for Logs.

    See Also:
        `AWS Cloud Formation documentation for SubscriptionFilter
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-subscriptionfilter.html>`_
    """

    RESOURCE_TYPE = "AWS::Logs::SubscriptionFilter"

    Properties: SubscriptionFilterProperties = attrib(
        factory=SubscriptionFilterProperties,
        converter=create_object_converter(SubscriptionFilterProperties),
    )



"""Raw representations of every data type in the AWS Route53Resolver service.

See Also:
    `AWS developer guide for Route53Resolver
    <https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "ResolverEndpoint",
    "ResolverEndpointProperties",
    "ResolverRule",
    "ResolverRuleProperties",
    "ResolverRuleAssociation",
    "ResolverRuleAssociationProperties",
]


@attrs(**ATTRSCONFIG)
class ResolverEndpointProperties(ResourceProperties):
    Direction = attrib(default=None)
    IpAddresses = attrib(default=None)
    Name = attrib(default=None)
    SecurityGroupIds = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ResolverEndpoint(Resource):
    """A Resolver Endpoint for Route53Resolver.

    See Also:
        `AWS Cloud Formation documentation for ResolverEndpoint
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53resolver-resolverendpoint.html>`_
    """

    RESOURCE_TYPE = "AWS::Route53Resolver::ResolverEndpoint"

    Properties: ResolverEndpointProperties = attrib(
        factory=ResolverEndpointProperties,
        converter=create_object_converter(ResolverEndpointProperties),
    )


@attrs(**ATTRSCONFIG)
class ResolverRuleProperties(ResourceProperties):
    DomainName = attrib(default=None)
    Name = attrib(default=None)
    ResolverEndpointId = attrib(default=None)
    RuleType = attrib(default=None)
    Tags = attrib(default=None)
    TargetIps = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ResolverRule(Resource):
    """A Resolver Rule for Route53Resolver.

    See Also:
        `AWS Cloud Formation documentation for ResolverRule
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53resolver-resolverrule.html>`_
    """

    RESOURCE_TYPE = "AWS::Route53Resolver::ResolverRule"

    Properties: ResolverRuleProperties = attrib(
        factory=ResolverRuleProperties,
        converter=create_object_converter(ResolverRuleProperties),
    )


@attrs(**ATTRSCONFIG)
class ResolverRuleAssociationProperties(ResourceProperties):
    Name = attrib(default=None)
    ResolverRuleId = attrib(default=None)
    VPCId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ResolverRuleAssociation(Resource):
    """A Resolver Rule Association for Route53Resolver.

    See Also:
        `AWS Cloud Formation documentation for ResolverRuleAssociation
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53resolver-resolverruleassociation.html>`_
    """

    RESOURCE_TYPE = "AWS::Route53Resolver::ResolverRuleAssociation"

    Properties: ResolverRuleAssociationProperties = attrib(
        factory=ResolverRuleAssociationProperties,
        converter=create_object_converter(ResolverRuleAssociationProperties),
    )



"""General-use classes to interact with the KinesisFirehose service through CloudFormation.

See Also:
    `AWS developer guide for KinesisFirehose
    <https://docs.aws.amazon.com/firehose/latest/dev/what-is-this-service.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import kinesisfirehose as _raw

# noinspection PyUnresolvedReferences
from .._raw.kinesisfirehose import *



"""General-use classes to interact with the ECS service through CloudFormation.

See Also:
    `AWS developer guide for ECS
    <https://docs.aws.amazon.com/AmazonECS/latest/developerguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import ecs as _raw

# noinspection PyUnresolvedReferences
from .._raw.ecs import *


# TODO specialised Service and TaskDefinition for use with fargate (which explicitly excludes some of the fields)

class TaskDefinition(_raw.TaskDefinition):
    __slots__ = []

    # Implement Naming
    # ----------------
    @property
    def name(self):
        # TaskDefinitions are assigned randomised names. However, if you put
        # them in a family, then all TaskDefinitions in that family are
        # considered to be different versions of the same TaskDefinition,
        # which as a group are referenced by the Family name. Hence the
        # Family name is the TaskDefinition name.
        return self.Properties["Family"]  # Might be default of None

    @name.setter
    def name(self, value: str):
        self.Properties["Family"] = value



"""General-use classes to interact with the Redshift service through CloudFormation.

See Also:
    `AWS developer guide for Redshift
    <https://docs.aws.amazon.com/redshift/latest/gsg/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import redshift as _raw

# noinspection PyUnresolvedReferences
from .._raw.redshift import *



"""General-use classes to interact with the KMS service through CloudFormation.

See Also:
    `AWS developer guide for KMS
    <https://docs.aws.amazon.com/kms/latest/developerguide/overview.html>`_
"""

from .._raw import kms as _raw

# noinspection PyUnresolvedReferences
from .._raw.kms import *


class Key(_raw.Key):
    __slots__ = []

    @property
    def name(self):
        # TODO #113 create an Alias instead
        raise NotImplementedError("Key.name cannot be set as a tag. We need to use an Alias.")



"""General-use classes to interact with the DirectoryService service through CloudFormation.

See Also:
    `AWS developer guide for DirectoryService
    <https://docs.aws.amazon.com/directoryservice/latest/admin-guide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import directoryservice as _raw

# noinspection PyUnresolvedReferences
from .._raw.directoryservice import *



"""Raw representations of every data type in the AWS Cognito service.

See Also:
    `AWS developer guide for Cognito
    <https://docs.aws.amazon.com/cognito/latest/developerguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "IdentityPool",
    "IdentityPoolProperties",
    "IdentityPoolRoleAttachment",
    "IdentityPoolRoleAttachmentProperties",
    "UserPool",
    "UserPoolProperties",
    "UserPoolClient",
    "UserPoolClientProperties",
    "UserPoolGroup",
    "UserPoolGroupProperties",
    "UserPoolUser",
    "UserPoolUserProperties",
    "UserPoolUserToGroupAttachment",
    "UserPoolUserToGroupAttachmentProperties",
]


@attrs(**ATTRSCONFIG)
class IdentityPoolProperties(ResourceProperties):
    AllowUnauthenticatedIdentities = attrib(default=None)
    CognitoEvents = attrib(default=None)
    CognitoIdentityProviders = attrib(default=None)
    CognitoStreams = attrib(default=None)
    DeveloperProviderName = attrib(default=None)
    IdentityPoolName = attrib(default=None)
    OpenIdConnectProviderARNs = attrib(default=None)
    PushSync = attrib(default=None)
    SamlProviderARNs = attrib(default=None)
    SupportedLoginProviders = attrib(default=None)


@attrs(**ATTRSCONFIG)
class IdentityPool(Resource):
    """A Identity Pool for Cognito.

    See Also:
        `AWS Cloud Formation documentation for IdentityPool
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-identitypool.html>`_
    """

    RESOURCE_TYPE = "AWS::Cognito::IdentityPool"

    Properties: IdentityPoolProperties = attrib(
        factory=IdentityPoolProperties,
        converter=create_object_converter(IdentityPoolProperties),
    )


@attrs(**ATTRSCONFIG)
class IdentityPoolRoleAttachmentProperties(ResourceProperties):
    IdentityPoolId = attrib(default=None)
    RoleMappings = attrib(default=None)
    Roles = attrib(default=None)


@attrs(**ATTRSCONFIG)
class IdentityPoolRoleAttachment(Resource):
    """A Identity Pool Role Attachment for Cognito.

    See Also:
        `AWS Cloud Formation documentation for IdentityPoolRoleAttachment
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-identitypoolroleattachment.html>`_
    """

    RESOURCE_TYPE = "AWS::Cognito::IdentityPoolRoleAttachment"

    Properties: IdentityPoolRoleAttachmentProperties = attrib(
        factory=IdentityPoolRoleAttachmentProperties,
        converter=create_object_converter(IdentityPoolRoleAttachmentProperties),
    )


@attrs(**ATTRSCONFIG)
class UserPoolProperties(ResourceProperties):
    AdminCreateUserConfig = attrib(default=None)
    AliasAttributes = attrib(default=None)
    AutoVerifiedAttributes = attrib(default=None)
    DeviceConfiguration = attrib(default=None)
    EmailConfiguration = attrib(default=None)
    EmailVerificationMessage = attrib(default=None)
    EmailVerificationSubject = attrib(default=None)
    LambdaConfig = attrib(default=None)
    MfaConfiguration = attrib(default=None)
    Policies = attrib(default=None)
    Schema = attrib(default=None)
    SmsAuthenticationMessage = attrib(default=None)
    SmsConfiguration = attrib(default=None)
    SmsVerificationMessage = attrib(default=None)
    UsernameAttributes = attrib(default=None)
    UserPoolName = attrib(default=None)
    UserPoolTags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class UserPool(Resource):
    """A User Pool for Cognito.

    See Also:
        `AWS Cloud Formation documentation for UserPool
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpool.html>`_
    """

    RESOURCE_TYPE = "AWS::Cognito::UserPool"

    Properties: UserPoolProperties = attrib(
        factory=UserPoolProperties,
        converter=create_object_converter(UserPoolProperties),
    )


@attrs(**ATTRSCONFIG)
class UserPoolClientProperties(ResourceProperties):
    ClientName = attrib(default=None)
    ExplicitAuthFlows = attrib(default=None)
    GenerateSecret = attrib(default=None)
    ReadAttributes = attrib(default=None)
    RefreshTokenValidity = attrib(default=None)
    UserPoolId = attrib(default=None)
    WriteAttributes = attrib(default=None)


@attrs(**ATTRSCONFIG)
class UserPoolClient(Resource):
    """A User Pool Client for Cognito.

    See Also:
        `AWS Cloud Formation documentation for UserPoolClient
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolclient.html>`_
    """

    RESOURCE_TYPE = "AWS::Cognito::UserPoolClient"

    Properties: UserPoolClientProperties = attrib(
        factory=UserPoolClientProperties,
        converter=create_object_converter(UserPoolClientProperties),
    )


@attrs(**ATTRSCONFIG)
class UserPoolGroupProperties(ResourceProperties):
    Description = attrib(default=None)
    GroupName = attrib(default=None)
    Precedence = attrib(default=None)
    RoleArn = attrib(default=None)
    UserPoolId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class UserPoolGroup(Resource):
    """A User Pool Group for Cognito.

    See Also:
        `AWS Cloud Formation documentation for UserPoolGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolgroup.html>`_
    """

    RESOURCE_TYPE = "AWS::Cognito::UserPoolGroup"

    Properties: UserPoolGroupProperties = attrib(
        factory=UserPoolGroupProperties,
        converter=create_object_converter(UserPoolGroupProperties),
    )


@attrs(**ATTRSCONFIG)
class UserPoolUserProperties(ResourceProperties):
    DesiredDeliveryMediums = attrib(default=None)
    ForceAliasCreation = attrib(default=None)
    MessageAction = attrib(default=None)
    UserAttributes = attrib(default=None)
    Username = attrib(default=None)
    UserPoolId = attrib(default=None)
    ValidationData = attrib(default=None)


@attrs(**ATTRSCONFIG)
class UserPoolUser(Resource):
    """A User Pool User for Cognito.

    See Also:
        `AWS Cloud Formation documentation for UserPoolUser
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpooluser.html>`_
    """

    RESOURCE_TYPE = "AWS::Cognito::UserPoolUser"

    Properties: UserPoolUserProperties = attrib(
        factory=UserPoolUserProperties,
        converter=create_object_converter(UserPoolUserProperties),
    )


@attrs(**ATTRSCONFIG)
class UserPoolUserToGroupAttachmentProperties(ResourceProperties):
    GroupName = attrib(default=None)
    Username = attrib(default=None)
    UserPoolId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class UserPoolUserToGroupAttachment(Resource):
    """A User Pool User To Group Attachment for Cognito.

    See Also:
        `AWS Cloud Formation documentation for UserPoolUserToGroupAttachment
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolusertogroupattachment.html>`_
    """

    RESOURCE_TYPE = "AWS::Cognito::UserPoolUserToGroupAttachment"

    Properties: UserPoolUserToGroupAttachmentProperties = attrib(
        factory=UserPoolUserToGroupAttachmentProperties,
        converter=create_object_converter(UserPoolUserToGroupAttachmentProperties),
    )



"""Raw representations of every data type in the AWS ApplicationAutoScaling service.

See Also:
    `AWS developer guide for ApplicationAutoScaling
    <https://docs.aws.amazon.com/autoscaling/application/APIReference/Welcome.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "ScalableTarget",
    "ScalableTargetProperties",
    "ScalingPolicy",
    "ScalingPolicyProperties",
]


@attrs(**ATTRSCONFIG)
class ScalableTargetProperties(ResourceProperties):
    MaxCapacity = attrib(default=None)
    MinCapacity = attrib(default=None)
    ResourceId = attrib(default=None)
    RoleARN = attrib(default=None)
    ScalableDimension = attrib(default=None)
    ScheduledActions = attrib(default=None)
    ServiceNamespace = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ScalableTarget(Resource):
    """A Scalable Target for ApplicationAutoScaling.

    See Also:
        `AWS Cloud Formation documentation for ScalableTarget
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalabletarget.html>`_
    """

    RESOURCE_TYPE = "AWS::ApplicationAutoScaling::ScalableTarget"

    Properties: ScalableTargetProperties = attrib(
        factory=ScalableTargetProperties,
        converter=create_object_converter(ScalableTargetProperties),
    )


@attrs(**ATTRSCONFIG)
class ScalingPolicyProperties(ResourceProperties):
    PolicyName = attrib(default=None)
    PolicyType = attrib(default=None)
    ResourceId = attrib(default=None)
    ScalableDimension = attrib(default=None)
    ScalingTargetId = attrib(default=None)
    ServiceNamespace = attrib(default=None)
    StepScalingPolicyConfiguration = attrib(default=None)
    TargetTrackingScalingPolicyConfiguration = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ScalingPolicy(Resource):
    """A Scaling Policy for ApplicationAutoScaling.

    See Also:
        `AWS Cloud Formation documentation for ScalingPolicy
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalingpolicy.html>`_
    """

    RESOURCE_TYPE = "AWS::ApplicationAutoScaling::ScalingPolicy"

    Properties: ScalingPolicyProperties = attrib(
        factory=ScalingPolicyProperties,
        converter=create_object_converter(ScalingPolicyProperties),
    )



"""General-use classes to interact with the IoT service through CloudFormation.

See Also:
    `AWS developer guide for IoT
    <https://docs.aws.amazon.com/iot/latest/developerguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import iot as _raw

# noinspection PyUnresolvedReferences
from .._raw.iot import *



"""Raw representations of every data type in the AWS IoT1Click service.

See Also:
    `AWS developer guide for IoT1Click
    <https://docs.aws.amazon.com/iot-1-click/latest/developerguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Device",
    "DeviceProperties",
    "Placement",
    "PlacementProperties",
    "Project",
    "ProjectProperties",
]


@attrs(**ATTRSCONFIG)
class DeviceProperties(ResourceProperties):
    DeviceId = attrib(default=None)
    Enabled = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Device(Resource):
    """A Device for IoT1Click.

    See Also:
        `AWS Cloud Formation documentation for Device
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot1click-device.html>`_
    """

    RESOURCE_TYPE = "AWS::IoT1Click::Device"

    Properties: DeviceProperties = attrib(
        factory=DeviceProperties,
        converter=create_object_converter(DeviceProperties),
    )


@attrs(**ATTRSCONFIG)
class PlacementProperties(ResourceProperties):
    AssociatedDevices = attrib(default=None)
    Attributes = attrib(default=None)
    PlacementName = attrib(default=None)
    ProjectName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Placement(Resource):
    """A Placement for IoT1Click.

    See Also:
        `AWS Cloud Formation documentation for Placement
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot1click-placement.html>`_
    """

    RESOURCE_TYPE = "AWS::IoT1Click::Placement"

    Properties: PlacementProperties = attrib(
        factory=PlacementProperties,
        converter=create_object_converter(PlacementProperties),
    )


@attrs(**ATTRSCONFIG)
class ProjectProperties(ResourceProperties):
    Description = attrib(default=None)
    PlacementTemplate = attrib(default=None)
    ProjectName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Project(Resource):
    """A Project for IoT1Click.

    See Also:
        `AWS Cloud Formation documentation for Project
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot1click-project.html>`_
    """

    RESOURCE_TYPE = "AWS::IoT1Click::Project"

    Properties: ProjectProperties = attrib(
        factory=ProjectProperties,
        converter=create_object_converter(ProjectProperties),
    )



"""Raw representations of every data type in the AWS DocDB service.

See Also:
    `AWS developer guide for DocDB
    <https://docs.aws.amazon.com/documentdb/latest/developerguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "DBCluster",
    "DBClusterProperties",
    "DBClusterParameterGroup",
    "DBClusterParameterGroupProperties",
    "DBInstance",
    "DBInstanceProperties",
    "DBSubnetGroup",
    "DBSubnetGroupProperties",
]


@attrs(**ATTRSCONFIG)
class DBClusterProperties(ResourceProperties):
    AvailabilityZones = attrib(default=None)
    BackupRetentionPeriod = attrib(default=None)
    DBClusterIdentifier = attrib(default=None)
    DBClusterParameterGroupName = attrib(default=None)
    DBSubnetGroupName = attrib(default=None)
    EngineVersion = attrib(default=None)
    KmsKeyId = attrib(default=None)
    MasterUsername = attrib(default=None)
    MasterUserPassword = attrib(default=None)
    Port = attrib(default=None)
    PreferredBackupWindow = attrib(default=None)
    PreferredMaintenanceWindow = attrib(default=None)
    SnapshotIdentifier = attrib(default=None)
    StorageEncrypted = attrib(default=None)
    Tags = attrib(default=None)
    VpcSecurityGroupIds = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DBCluster(Resource):
    """A Db Cluster for DocDB.

    See Also:
        `AWS Cloud Formation documentation for DBCluster
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-docdb-dbcluster.html>`_
    """

    RESOURCE_TYPE = "AWS::DocDB::DBCluster"

    Properties: DBClusterProperties = attrib(
        factory=DBClusterProperties,
        converter=create_object_converter(DBClusterProperties),
    )


@attrs(**ATTRSCONFIG)
class DBClusterParameterGroupProperties(ResourceProperties):
    Description = attrib(default=None)
    Family = attrib(default=None)
    Name = attrib(default=None)
    Parameters = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DBClusterParameterGroup(Resource):
    """A Db Cluster Parameter Group for DocDB.

    See Also:
        `AWS Cloud Formation documentation for DBClusterParameterGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-docdb-dbclusterparametergroup.html>`_
    """

    RESOURCE_TYPE = "AWS::DocDB::DBClusterParameterGroup"

    Properties: DBClusterParameterGroupProperties = attrib(
        factory=DBClusterParameterGroupProperties,
        converter=create_object_converter(DBClusterParameterGroupProperties),
    )


@attrs(**ATTRSCONFIG)
class DBInstanceProperties(ResourceProperties):
    AutoMinorVersionUpgrade = attrib(default=None)
    AvailabilityZone = attrib(default=None)
    DBClusterIdentifier = attrib(default=None)
    DBInstanceClass = attrib(default=None)
    DBInstanceIdentifier = attrib(default=None)
    PreferredMaintenanceWindow = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DBInstance(Resource):
    """A Db Instance for DocDB.

    See Also:
        `AWS Cloud Formation documentation for DBInstance
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-docdb-dbinstance.html>`_
    """

    RESOURCE_TYPE = "AWS::DocDB::DBInstance"

    Properties: DBInstanceProperties = attrib(
        factory=DBInstanceProperties,
        converter=create_object_converter(DBInstanceProperties),
    )


@attrs(**ATTRSCONFIG)
class DBSubnetGroupProperties(ResourceProperties):
    DBSubnetGroupDescription = attrib(default=None)
    DBSubnetGroupName = attrib(default=None)
    SubnetIds = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DBSubnetGroup(Resource):
    """A Db Subnet Group for DocDB.

    See Also:
        `AWS Cloud Formation documentation for DBSubnetGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-docdb-dbsubnetgroup.html>`_
    """

    RESOURCE_TYPE = "AWS::DocDB::DBSubnetGroup"

    Properties: DBSubnetGroupProperties = attrib(
        factory=DBSubnetGroupProperties,
        converter=create_object_converter(DBSubnetGroupProperties),
    )



"""Raw representations of every data type in the AWS Glue service.

See Also:
    `AWS developer guide for Glue
    <https://docs.aws.amazon.com/glue/latest/dg/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Classifier",
    "ClassifierProperties",
    "Connection",
    "ConnectionProperties",
    "Crawler",
    "CrawlerProperties",
    "Database",
    "DatabaseProperties",
    "DevEndpoint",
    "DevEndpointProperties",
    "Job",
    "JobProperties",
    "Partition",
    "PartitionProperties",
    "Table",
    "TableProperties",
    "Trigger",
    "TriggerProperties",
]


@attrs(**ATTRSCONFIG)
class ClassifierProperties(ResourceProperties):
    GrokClassifier = attrib(default=None)
    JsonClassifier = attrib(default=None)
    XMLClassifier = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Classifier(Resource):
    """A Classifier for Glue.

    See Also:
        `AWS Cloud Formation documentation for Classifier
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-glue-classifier.html>`_
    """

    RESOURCE_TYPE = "AWS::Glue::Classifier"

    Properties: ClassifierProperties = attrib(
        factory=ClassifierProperties,
        converter=create_object_converter(ClassifierProperties),
    )


@attrs(**ATTRSCONFIG)
class ConnectionProperties(ResourceProperties):
    CatalogId = attrib(default=None)
    ConnectionInput = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Connection(Resource):
    """A Connection for Glue.

    See Also:
        `AWS Cloud Formation documentation for Connection
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-glue-connection.html>`_
    """

    RESOURCE_TYPE = "AWS::Glue::Connection"

    Properties: ConnectionProperties = attrib(
        factory=ConnectionProperties,
        converter=create_object_converter(ConnectionProperties),
    )


@attrs(**ATTRSCONFIG)
class CrawlerProperties(ResourceProperties):
    Classifiers = attrib(default=None)
    Configuration = attrib(default=None)
    DatabaseName = attrib(default=None)
    Description = attrib(default=None)
    Name = attrib(default=None)
    Role = attrib(default=None)
    Schedule = attrib(default=None)
    SchemaChangePolicy = attrib(default=None)
    TablePrefix = attrib(default=None)
    Targets = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Crawler(Resource):
    """A Crawler for Glue.

    See Also:
        `AWS Cloud Formation documentation for Crawler
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-glue-crawler.html>`_
    """

    RESOURCE_TYPE = "AWS::Glue::Crawler"

    Properties: CrawlerProperties = attrib(
        factory=CrawlerProperties,
        converter=create_object_converter(CrawlerProperties),
    )


@attrs(**ATTRSCONFIG)
class DatabaseProperties(ResourceProperties):
    CatalogId = attrib(default=None)
    DatabaseInput = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Database(Resource):
    """A Database for Glue.

    See Also:
        `AWS Cloud Formation documentation for Database
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-glue-database.html>`_
    """

    RESOURCE_TYPE = "AWS::Glue::Database"

    Properties: DatabaseProperties = attrib(
        factory=DatabaseProperties,
        converter=create_object_converter(DatabaseProperties),
    )


@attrs(**ATTRSCONFIG)
class DevEndpointProperties(ResourceProperties):
    EndpointName = attrib(default=None)
    ExtraJarsS3Path = attrib(default=None)
    ExtraPythonLibsS3Path = attrib(default=None)
    NumberOfNodes = attrib(default=None)
    PublicKey = attrib(default=None)
    RoleArn = attrib(default=None)
    SecurityGroupIds = attrib(default=None)
    SubnetId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DevEndpoint(Resource):
    """A Dev Endpoint for Glue.

    See Also:
        `AWS Cloud Formation documentation for DevEndpoint
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-glue-devendpoint.html>`_
    """

    RESOURCE_TYPE = "AWS::Glue::DevEndpoint"

    Properties: DevEndpointProperties = attrib(
        factory=DevEndpointProperties,
        converter=create_object_converter(DevEndpointProperties),
    )


@attrs(**ATTRSCONFIG)
class JobProperties(ResourceProperties):
    AllocatedCapacity = attrib(default=None)
    Command = attrib(default=None)
    Connections = attrib(default=None)
    DefaultArguments = attrib(default=None)
    Description = attrib(default=None)
    ExecutionProperty = attrib(default=None)
    LogUri = attrib(default=None)
    MaxRetries = attrib(default=None)
    Name = attrib(default=None)
    Role = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Job(Resource):
    """A Job for Glue.

    See Also:
        `AWS Cloud Formation documentation for Job
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-glue-job.html>`_
    """

    RESOURCE_TYPE = "AWS::Glue::Job"

    Properties: JobProperties = attrib(
        factory=JobProperties,
        converter=create_object_converter(JobProperties),
    )


@attrs(**ATTRSCONFIG)
class PartitionProperties(ResourceProperties):
    CatalogId = attrib(default=None)
    DatabaseName = attrib(default=None)
    PartitionInput = attrib(default=None)
    TableName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Partition(Resource):
    """A Partition for Glue.

    See Also:
        `AWS Cloud Formation documentation for Partition
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-glue-partition.html>`_
    """

    RESOURCE_TYPE = "AWS::Glue::Partition"

    Properties: PartitionProperties = attrib(
        factory=PartitionProperties,
        converter=create_object_converter(PartitionProperties),
    )


@attrs(**ATTRSCONFIG)
class TableProperties(ResourceProperties):
    CatalogId = attrib(default=None)
    DatabaseName = attrib(default=None)
    TableInput = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Table(Resource):
    """A Table for Glue.

    See Also:
        `AWS Cloud Formation documentation for Table
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-glue-table.html>`_
    """

    RESOURCE_TYPE = "AWS::Glue::Table"

    Properties: TableProperties = attrib(
        factory=TableProperties,
        converter=create_object_converter(TableProperties),
    )


@attrs(**ATTRSCONFIG)
class TriggerProperties(ResourceProperties):
    Actions = attrib(default=None)
    Description = attrib(default=None)
    Name = attrib(default=None)
    Predicate = attrib(default=None)
    Schedule = attrib(default=None)
    Type = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Trigger(Resource):
    """A Trigger for Glue.

    See Also:
        `AWS Cloud Formation documentation for Trigger
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-glue-trigger.html>`_
    """

    RESOURCE_TYPE = "AWS::Glue::Trigger"

    Properties: TriggerProperties = attrib(
        factory=TriggerProperties,
        converter=create_object_converter(TriggerProperties),
    )



"""General-use classes to interact with the RAM service through CloudFormation.

See Also:
    `AWS developer guide for RAM
    <https://docs.aws.amazon.com/ram/latest/userguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import ram as _raw

# noinspection PyUnresolvedReferences
from .._raw.ram import *



"""General-use classes to interact with the SecretsManager service through CloudFormation.

See Also:
    `AWS developer guide for SecretsManager
    <https://docs.aws.amazon.com/secretsmanager/latest/userguide/intro.html>`_
"""

from .._raw import secretsmanager as _raw

# noinspection PyUnresolvedReferences
from .._raw.secretsmanager import *


class Secret(_raw.Secret):
    __slots__ = []

    # Implement Naming
    # ----------------
    @property
    def name(self):
        return self.Properties["Name"]  # Might be default of None

    @name.setter
    def name(self, value: str):
        self.Properties["Name"] = value



"""General-use classes to interact with the Kinesis service through CloudFormation.

See Also:
    `AWS developer guide for Kinesis
    <https://docs.aws.amazon.com/guardduty/latest/ug/what-is-guardduty.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import kinesis as _raw

# noinspection PyUnresolvedReferences
from .._raw.kinesis import *



"""Raw representations of every data type in the AWS RAM service.

See Also:
    `AWS developer guide for RAM
    <https://docs.aws.amazon.com/ram/latest/userguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "ResourceShare",
    "ResourceShareProperties",
]


@attrs(**ATTRSCONFIG)
class ResourceShareProperties(ResourceProperties):
    AllowExternalPrincipals = attrib(default=None)
    Name = attrib(default=None)
    Principals = attrib(default=None)
    ResourceArns = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ResourceShare(Resource):
    """A Resource Share for RAM.

    See Also:
        `AWS Cloud Formation documentation for ResourceShare
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ram-resourceshare.html>`_
    """

    RESOURCE_TYPE = "AWS::RAM::ResourceShare"

    Properties: ResourceShareProperties = attrib(
        factory=ResourceShareProperties,
        converter=create_object_converter(ResourceShareProperties),
    )



"""Core classes for composing AWS Cloud Formation Stacks."""

import copy
import re
import textwrap
from itertools import chain
from typing import Any
from typing import Dict
from typing import Iterator
from typing import List
from typing import Optional

import attr
import yaml
import yaml.resolver
from attr import Attribute
from attr import attrib
from attr import attrs

from . import _about
from .exceptions import StackMergeError
from .yaml import AmazonCFNDumper
from .yaml import CustomYamlObject

# TODO rename "AWS attribute" to "CloudFormation attribute" (or YAML attribute) everywhere ?

#: Arguments to `attrs` decorator for creating a CloudFormation object,
#: as a subclass of CustomYamlObject.
#:
#: Some type checkers (eg PyCharm) implement attrs-based type-checking by
#: detecting the use of the @attrs decorator and then examining the decorated
#: class. These type checkers break when we wrap attrs() in our own extensions,
#: so we instead provide our standard configuration as a set of arguments
ATTRSCONFIG = dict(
    # TODO Consider whether we should have comparison. It's not normally needed, but it wouldnt hurt either
    cmp=False,
    # Only allow known attributes. This requires all parent classes to also
    # have __slots__ defined.
    slots=True,
    # Create a constructor with only keyword parameters
    init=True, kw_only=True,
)


def create_object_converter(ObjectClass):
    """Create a converter for attributes using this AWSObject class"""

    def convert_to_object(value):
        """Convert a possible dictionary argument into it's object type"""
        # Convert a dictionary
        if isinstance(value, dict):
            return ObjectClass(**value)

        # Leave all other objects untouched. If they have an invalid type,
        # that will be picked up by a validator later.
        return value

    return convert_to_object


# TODO create some prototypes or helper functions for creating attribs().
# prototype_aws_attribute: int = attrib(default=None)
# _prototype_internal_attribute: int = attrib(default=None, init=False)


@attrs(**ATTRSCONFIG)
class AWSObject(CustomYamlObject):
    """Base class to represent any dictionary-like object from AWS Cloud Formation.

    Cloud Formation attributes are stored as Python attributes on the object,
    using the `attrs` library.

    Any public Python instance attribute is considered to be a Cloud
    Formation attribute. By contrast, a private Python attribute (ie. named
    with a leading underscore) is considered to be an internal attribute and
    is not exported to Cloud Formation.

    If an attribute is set to None it is considered to not be set for cloud
    formation. It is still returned when accessed in code, but won't be
    exported.

    Attributes are exported in the order they are declared.
    """

    # TODO rename to BaseObject and make this an AWS-specific module for context

    # TODO use attrs metadata to mark non-exported attributes

    # TODO beware we have changed the semantics of hasattr with our `attrs` reimplementation. Nowadays the attr is always there, but has a signal value to say it is absent

    # FIXME validate behaviour when we have subclassed objects with attribs defined at multiple levels (all should be included)
    #   - can't set invalid attributes => slots should be set all the way up

    # FIXME convert existing "safe" getattr code to simple attribute lookup

    # Attribute Access
    # ----------------

    def __setattr__(self, key: str, value: Any):
        # attrs only applies converters and validators at initialisation, so
        # we use the existing attribute declaration to also execute them when
        # attributes are set
        # TODO push this upstream to the attrs library, controlled by a flag on @attrs ?

        # Convert value first, if necessary
        try:
            data: Attribute = attr.fields_dict(self.__class__)[key]
        except KeyError:
            # This attribute is not actually an attrs attribute. Let the
            # superclass deal with that...
            pass
        else:
            if data.converter:
                value = data.converter(value)

        # Set the value normally
        super(AWSObject, self).__setattr__(key, value)

        # Run per-attribute and per-class validation
        attr.validate(self)

    def _is_internal_attribute(self, name: str) -> bool:
        """Whether this attribute name corresponds to an internal attribute for this object.

        This is effectively the inverse of _is_cfn_attribute().
        """
        # TODO use attribs metadata
        # Internal attributes all start with an underscore
        return name.startswith("_")

    def _is_cfn_attribute(self, name: str) -> bool:
        """Whether this attribute name corresponds to a known CloudFormation attribute for this object."""
        # TODO use attribs metadata
        # A CloudFormation attribute is an attribute that's not internal.
        return not self._is_internal_attribute(name)

    def is_attribute_set(self, name: str) -> bool:
        """Whether this attribute has a valid value."""
        # Currently we use None as a signal value (and default value in
        # attrib() definitions) to indicate that the attribute is not set.

        # TODO If we ever encounter a need to use None as a real value,
        # have some attrib() metadata that is used to indicate the
        # alternate signal value for not-set (eg. "fc-notset-signal-value")

        return getattr(self, name, None) is not None

    # Container-Like Access For CloudFormation Attributes
    # ---------------------------------------------------
    # TODO implement other container functions: __reversed__, __contains__
    # TODO probably need __reversed__ otherwise an attempt to reverse the object will break like crazy (int item lookups)

    def __getitem__(self, item: str) -> Any:
        """Get CloudFormation attributes in a dictionary-like manner."""
        if self._is_cfn_attribute(item):
            try:
                return getattr(self, item)
            except AttributeError as ex:
                raise KeyError(str(ex)) from ex
        else:
            raise KeyError(
                "'{}' is not a CloudFormation attribute, and cannot be retrieved with the dictionary interface".format(
                    item))

    def __setitem__(self, key: str, value: Any):
        """Set CloudFormation attributes in a dictionary-like manner."""
        if self._is_cfn_attribute(key):
            setattr(self, key, value)
        else:
            raise KeyError(
                "'{}' is not a CloudFormation attribute, and cannot be set with the dictionary interface".format(key))

    def __iter__(self) -> Iterator[str]:
        # We treat the object like a dictionary for iteration. This means
        # we return a sorted list of CloudFormation attribute names.

        # Sorting by declaration order does not work with subclassing. For
        # now, we achieve this by having the sorted subclass override the
        # behaviour of this function.
        #
        # If there turns out to be a real need for more control, we can
        # re-introduce partial sort-order declaration like we used to
        # have, controlled by a class constant.

        # noinspection PyUnresolvedReferences
        for attrib in self.__attrs_attrs__:
            if self._is_cfn_attribute(attrib.name):
                yield attrib.name

    def __len__(self) -> int:
        # The length of the object is the number of attributes currently set,
        # in order to match the result from __iter__
        return len(list(iter(self)))

    # Export Data
    # -----------

    # TODO change param name to prevent namespace clash
    def export(self, format: str = "yaml") -> str:
        """Export this AWS object as CloudFormation in the specified format."""
        # TODO document the formats. 'json' or 'yaml'
        if format == "yaml":
            return yaml.dump_all(
                [self],
                Dumper=AmazonCFNDumper,
                default_flow_style=False,
                explicit_start=True,
            )
        else:
            raise ValueError("Export format '{}' is unknown".format(format))

    def as_yaml_node(self, dumper: yaml.Dumper) -> yaml.Node:
        # Ideally, we would create a tag that contains the object name
        # (eg. "!Bucket") for completeness. Unfortunately, there is no way
        # to then prevent the YAML dumper from printing tags unless it
        # determines that the tag is implicit (ie. the default tag for a
        # mapping node is the tag being used), so we end up just using the
        # default tag.
        # TODO investigate hacking a Dumper subclass like we did for aliasing
        tag = yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG

        # Get all cloud formation attributes that are set, in sorted order
        attributes = [(key, self[key]) for key in self if self.is_attribute_set(key)]

        # Create neater YAML by filtering out empty blocks at this level
        attributes = [(key, value) for key, value in attributes if is_non_empty_attribute(value)]

        # Create neater YAML by filtering out empty entries in sub-lists
        attributes = [(key, remove_empty_values_from_attribute(value)) for key, value in attributes]

        # Represent this object as a mapping of it's AWS attributes.
        # Note that `represent_mapping` works on a list of 2-tuples, not a map!
        return dumper.represent_mapping(tag, attributes)


def remove_empty_values_from_attribute(data):
    """If this attribute is a list or dictionary, return a copy with empty entries recursively removed."""
    if isinstance(data, list):
        cleaned = map(remove_empty_values_from_attribute, data)
        cleaned = filter(is_non_empty_attribute, cleaned)
        return list(cleaned)
    elif isinstance(data, dict):
        cleaned = {key: remove_empty_values_from_attribute(value) for key, value in data.items()}
        cleaned = {key: value for key, value in cleaned.items() if is_non_empty_attribute(value)}
        return dict(cleaned)

    # Don't modify other types
    return data


def is_non_empty_attribute(data):
    """Is this attribute of an AWS object non-empty?

    The purpose of this function is to provide clean output by not exporting
    data that has no semantic meaning. It is worth noting that empty-ish
    values like None and "" do actually have a meaning.

    A non-empty object is defined recursively as:
      - not a block-like object
      - a block-like object that contains at least one non-empty attribute.
    """
    if isinstance(data, (tuple, list)):
        for value in data:
            if is_non_empty_attribute(value):
                return True
        return False
    elif isinstance(data, dict):
        for value in data.values():
            if is_non_empty_attribute(value):
                return True
        return False
    elif isinstance(data, AWSObject):
        for key in data:
            if data.is_attribute_set(key) and is_non_empty_attribute(data[key]):
                return True
        return False

    # Other types of object are never empty
    return True


class _EmptyList(CustomYamlObject):
    """Represents an immutable YAML-isable list with no entries."""

    def as_yaml_node(self, dumper):
        return dumper.represent_list([])


#: Signal value for an empty list.
#:
#: Normally empty lists are trimmed from the AWSObject's export, in
#: order to produce cleaner YAML. This special object will allow you to
#: force the export of an empty list.
EMPTY_LIST = _EmptyList()


class _EmptyDict(CustomYamlObject):
    """Represents an immutable YAML-isable dictionary with no items."""

    def as_yaml_node(self, dumper):
        return dumper.represent_dict({})


#: Signal value for an empty dictionary.
#:
#: Normally empty dictionaries are trimmed from the AWSObject's export, in
#: order to produce cleaner YAML. This special object will allow you to
#: force the export of an empty dictionary.
EMPTY_DICT = _EmptyDict()


@attrs(**ATTRSCONFIG)
class Stack(AWSObject):
    """Represents a CloudFormation Stack, the top-level template object.

    See http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-anatomy.html
    """

    # These are declared in the order recommended in the documentation
    # TODO can probably nail down the value types for these dicts - Resource, Output, etc.
    AWSTemplateFormatVersion: str = attrib(default="2010-09-09")
    Description: Optional[str] = attrib(default=None)
    Metadata: Dict[str, Any] = attrib(factory=dict)
    Parameters: Dict[str, Any] = attrib(factory=dict)
    Mappings: Dict[str, Any] = attrib(factory=dict)
    Conditions: Dict[str, Any] = attrib(factory=dict)
    Transform: Optional[str] = attrib(default=None)
    Resources: Dict[str, Any] = attrib(factory=dict)
    Outputs: Dict[str, Any] = attrib(factory=dict)

    def __attrs_post_init__(self):
        # Set standard Metadata
        self.Metadata["FlyingCircus"] = {
            "version": _about.__version__,
        }

    def as_yaml_node(self, dumper):
        dumper.cfn_stack = self
        return super().as_yaml_node(dumper)

    def get_logical_name(self, resource, resources_only=False):
        """Get the logical name used for this object in this stack.

        Raises ValueError if the object is not in this stack.
        """
        # Pseudo Parameters are a special case. They can be thought of as
        # implicitly part of the current stack, so we support finding them
        # through this function
        if not resources_only and isinstance(resource, PseudoParameter):
            return str(resource)

        # The lazy and non-performant approach is to iterate through all
        # the objects in this stack until we find the supplied object.
        # That should do for now, really.
        matches = [name for name, data in self.Resources.items() if data is resource]
        if not resources_only:
            matches.extend([name for name, data in self.Parameters.items() if data is resource])

        if len(matches) > 1:
            raise ValueError("Object has multiple names in this stack: {}".format(resource))
        elif len(matches) == 1:
            return matches[0]
        raise ValueError("Object is not part of this stack: {}".format(resource))

    def merge_stack(self, other):
        """Add a reference in this stack to all objects from the supplied stack.

        Return the current stack for call-chaining purposes.
        """
        # Check that we aren't using incompatible versions. Note that the
        # 'Transform' attribute is currently used to store the version of
        # the Serverless Application Model we are using (if any), so it
        # counts as a version too.
        if self.AWSTemplateFormatVersion != other.AWSTemplateFormatVersion:
            raise StackMergeError(
                "This template has a different template version ({}) to the "
                "other template ({})".format(
                    self.AWSTemplateFormatVersion, other.AWSTemplateFormatVersion
                )
            )
        if self.Transform is not None and \
                other.Transform is not None and \
                self.Transform != other.Transform:
            raise StackMergeError(
                "This template has a different Serverless Application Model "
                "Transform version ({}) to the other template ({})".format(
                    self.Transform, other.Transform
                )
            )

        # Check for output's with a repeated name. The export name can be
        # different from the logical name of the Output in the stack, so it
        # wouldn't otherwise be checked.
        #
        # Duplicated export names are picked up by cloud formation when we
        # import the template, but when merging stacks it is a lot more
        # helpful to catch these errors early
        existing_exports = {getattr(output, "Export", {}).get("Name", "") for output in self.Outputs.values()}
        new_exports = {getattr(output, "Export", {}).get("Name", "") for output in other.Outputs.values()}
        shared_exports = existing_exports.intersection(new_exports)
        shared_exports.discard("")
        if shared_exports:
            raise StackMergeError(
                "The target stack already has exports named {}".format(", ".join(sorted(shared_exports)))
            )

        # Copy the name and reference for the relevant item types
        for item_type in ["Resources", "Parameters", "Outputs"]:
            existing_items = self[item_type]
            new_items = other[item_type]
            for name, value in new_items.items():
                if name in existing_items:
                    raise StackMergeError(
                        "{} in this stack already has an item with the logical name {}".format(item_type, name)
                    )
                existing_items[name] = value

        # Copy across metadata from other sources.
        for name, value in other.Metadata.items():
            if name == "FlyingCircus":
                continue
            if name in self.Metadata:
                raise StackMergeError(
                    "Metadata in this stack already has an item with the logical name {}".format(name)
                )
            self.Metadata[name] = value

        return self

    def with_prefixed_names(self, prefix):
        """Create a new stack which has the same objects as the current stack,
        but with the supplied prefix added to the logical and external names
        of all appropriate objects.

        Return the new stack.
        """
        # TODO Add checks for logical name format when we set items, as well as here
        # FIXME #43 Copy across Metadata. i don't think we should prefix it.

        # Filter out ridiculous prefixes before they cause subtle damage
        if not isinstance(prefix, str):
            raise TypeError("Prefix should be a string")
        if not prefix:
            raise ValueError("Prefix should not be empty")
        if not re.fullmatch(r"[A-Z]\w*", prefix):
            raise ValueError(
                "Prefix should have alphanumeric or underscore characters, "
                "beginning with an uppercase character: '{}'".format(prefix)
            )

        # Create a new stack with copies of the static text fields
        new_stack = Stack(
            AWSTemplateFormatVersion=self.AWSTemplateFormatVersion,
            Transform=self.Transform,
        )

        # Modify the description to refer to the prefix. This is a bit hacky,
        # but for our use cases we don't expect the description to be
        # retained in final output, so it's good enough for now
        if self.Description:
            new_stack.Description = prefix + ": " + self.Description
        else:
            new_stack.Description = prefix

        # Insert all references to standard existing objects into new stack, with modified names
        # Copy the name and reference for the relevant item types
        for item_type in ["Resources", "Parameters"]:
            new_stack[item_type] = new_items = {}
            for name, value in self[item_type].items():
                new_items[prefix + name] = value

        # Create copies of all Outputs, with modified Export name where appropriate
        if hasattr(self, "Outputs"):
            new_stack.Outputs = {}
            for name, output in self.Outputs.items():
                # Beware that a deep copy may wreck any intrinsic functions
                # we have attached (a common use case), which is undesirable.
                #
                # OTOH, a shallow copy means that any modifications to the
                # attribute's values (like Export) will affect the original,
                # which is also undesirable. We just need to remember to
                # allow for this
                output = copy.copy(output)
                if getattr(output, "Export", {}).get("Name", ""):
                    output.Export = {"Name": prefix + output.Export["Name"]}

                new_stack.Outputs[prefix + name] = output

        return new_stack

    def tag(self, tags=None, tag_derived_resources=True, **more_tags):
        """Apply tags to all resources in this stack, where they are supported.

        Parameters:
            tags (dict): Key-value pairs representing tags to apply to the
             resource.
            more_tags: Additional key-value pairs representing tags
             to apply to the resource, specified as keyword arguments.
             It is not defined which tag is actually applied in the event
             of duplicates.
            tag_derived_resources (bool): Whether to attempt to apply the
             same tags to derived resources, using Cloud Formation
             (eg. EC2 instances started by an auto-scaling group).

        See Also:
            `AWS documentation on resource tagging
            <https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-resource-tags.html>`_
        """
        # TODO naming ambiguity - `tag` sounds like it could just mean "tag this single object"

        # Handle multiple ways of passing tags
        if tags is None:
            tags = more_tags
        else:
            tags = dict(tags)
            tags.update(more_tags)

        # Simply apply the tags to each resource
        # TODO force the type for Resources to always be a resource, then can simplify this checking
        for resource in self.Resources.values():
            if hasattr(resource, "tag") and callable(resource.tag):
                # Assume this is a Resource-like object
                resource.tag(tags=tags, tag_derived_resources=tag_derived_resources)


@attrs(**ATTRSCONFIG)
class Parameter(AWSObject):
    """Represents a CloudFormation Parameter.

    https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/parameters-section-structure.html
    """

    Type: Optional[str] = attrib(default=None)
    AllowedPattern: Optional[str] = attrib(default=None)
    AllowedValues: List[Any] = attrib(factory=list)
    ConstraintDescription: Optional[str] = attrib(default=None)
    Default: Optional[Any] = attrib(default=None)
    Description: Optional[str] = attrib(default=None)
    MaxLength: Optional[int] = attrib(default=None)
    MaxValue: Optional[int] = attrib(default=None)  # TODO this could also ]be a float
    MinLength: Optional[int] = attrib(default=None)
    MinValue: Optional[int] = attrib(default=None)  # TODO this could also be float
    NoEcho: Optional[bool] = attrib(default=None)

    # TODO check Type is set to a valid value?
    # TODO check MinLength/MaxLength are set to a valid integer value?


class PseudoParameter(str):
    """Represents an AWS-provided pseudo parameter.

    This is simply a string with extra mixin behaviour.

    See http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/pseudo-parameter-reference.html
    """

    #: Set of all AWS pseudo-parameters
    ALL = set()

    # TODO if we try to use it as a YAML scalar, then implicitly make it be a Ref? Do with as_yaml_node()

    @classmethod
    def _create_standard_parameter(cls, name):
        obj = cls(name)
        cls.ALL.add(obj)
        return obj


# TODO not convinced about the names of the constants, or about putting them in this namespace
AWS_AccountId = PseudoParameter._create_standard_parameter("AWS::AccountId")
AWS_NotificationARNs = PseudoParameter._create_standard_parameter("AWS::NotificationARNs")
AWS_NoValue = PseudoParameter._create_standard_parameter("AWS::NoValue")
AWS_Partition = PseudoParameter._create_standard_parameter("AWS::Partition")
AWS_Region = PseudoParameter._create_standard_parameter("AWS::Region")
AWS_StackId = PseudoParameter._create_standard_parameter("AWS::StackId")
AWS_StackName = PseudoParameter._create_standard_parameter("AWS::StackName")
AWS_URLSuffix = PseudoParameter._create_standard_parameter("AWS::URLSuffix")


@attrs(**ATTRSCONFIG)
class Resource(AWSObject):
    """Represents a CloudFormation Resource in a Stack.

    See http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/resources-section-structure.html
    and http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-product-attribute-reference.html

    Subclasses need to:
     * Use the @attrs decorator
     * Define RESOURCE_TYPE
     * Define a ResourceProperties subclass and the Properties attribute
       using fields specific to that resource type
     * Define attributes for CreationPolicy and UpdatePolicy, if relevant.
    """

    # `CreationPolicy` is defined as a Resource-level attribute in
    # CloudFormation, but is currently only valid for a few resource types.
    # Therefore we leave it to concrete subclasses to defined the attribute
    # when it exists.
    #
    # See http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-attribute-creationpolicy.html
    #
    # CreationPolicy: Dict[str, Any] = attrib(factory=dict)

    DeletionPolicy: str = attrib(default=None)
    DependsOn: List[str] = attrib(factory=list)

    #: Note that Resource-level Metadata is not expected to be used by Flying
    #: Circus users, so it can't be set in the constructor. However, they can
    #: still modify it and set it as an attribute.
    Metadata: Dict[str, Any] = attrib(factory=dict, init=False)

    # `Properties` has a resource-type-specific Python type, so we leave it to
    # each concrete subtype to define the attribute appropriately.
    #
    # Properties: ResourceProperties = attrib(factory=ResourceProperties, converter=create_object_converter(ResourceProperties))

    # `Type` is defined as a Python property - see below
    #
    # Type: str = attrib(init=False)

    # `UpdatePolicy` is defined as a Resource-level attribute in
    # CloudFormation, but is currently only valid for a few resource types.
    # Therefore we leave it to concrete subclasses to define the attribute
    # when it exists.
    #
    # See http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-attribute-updatepolicy.html
    #
    # UpdatePolicy: Dict[str, Any] = attrib(factory=dict)

    UpdateReplacePolicy: str = attrib(default=None)

    #: Customised sort order for a Resource class. We need to implement
    #: special functionality to sort the attributes on a Resource, because
    #: they are defined in several different (non-standard) places.
    #:
    #: This list includes all the attributes that are defined on a
    #: CloudFormation Resource, even if they may not exist on every concrete
    #: Resource subclass
    _SORT_ORDER = [
        "Type", "DependsOn", "Metadata", "CreationPolicy", "UpdatePolicy", "UpdateReplacePolicy",
        "DeletionPolicy", "Properties",
    ]

    #: The AWS CloudFormation string for this resource's Type
    RESOURCE_TYPE = None

    #: The name of the property that tags are stored in (if any)
    TAG_PROPERTY = "Tags"

    # TODO implement a shortcut function for get_ref(), instead of having to bring in the fn.Ref function?

    def __attrs_post_init__(self):
        # Check that the resource type is specified
        try:
            _ = self.Type
        except AttributeError as ex:
            raise TypeError(ex)

        # Check that the Properties attribute has been defined correctly
        if not (hasattr(self, "Properties") and isinstance(self.Properties, (dict, ResourceProperties))):
            raise TypeError(
                "Concrete Resource class {} needs to define an attribute "
                "called `Properties`".format(self.__class__.__name__)
            )

    @property
    def Type(self) -> str:
        # The `Type` attribute is a class-specific fixed-value attribute.
        # It is coupled to the underlying concrete subclass via this class
        # constant.
        #
        # We can't achieve this functionality by declaring it as a normal
        # `attrs`-style attribute, so we implement it as a property and
        # override all the base-class functionality that is aware of
        # CloudFormation attributes.
        if not self.RESOURCE_TYPE:
            raise AttributeError(
                "Concrete Resource class {} needs to define the "
                "CloudFormation Type as a class variable called "
                "RESOURCE_TYPE".format(
                    self.__class__.__name__
                )
            )
        return self.RESOURCE_TYPE

    def __iter__(self) -> Iterator[str]:
        # Get the attributes for this Resource type
        attribs = set(super().__iter__())

        # Type is a class-specific fixed-value attribute, which means we
        # define it specially. We need to explicitly add it to the list
        # of attributes here
        attribs.add("Type")

        # Filter our custom sort order based on which fields actually exist
        # for this Resource class
        for name in self._SORT_ORDER:
            if name in attribs:
                yield name

    @property
    def is_retained(self) -> bool:
        """Whether the underlying physical resource will always be retained,
        regardless of any changes in the stack.

        This reflects whether *both* the underlying policies (`DeletionPolicy`
        and `UpdateReplacePolicy`) are set to "Retain". It is an effective way
        to ensure that irreplacable resources cannot be accidentally removed
        by the CloudFormation.

        Ensure you understand the cost implications of setting this flag.
        """
        return self.DeletionPolicy == "Retain" and self.UpdateReplacePolicy == "Retain"

    @is_retained.setter
    def is_retained(self, value: bool):
        if value:
            self.DeletionPolicy = "Retain"
            self.UpdateReplacePolicy = "Retain"
        else:
            self.DeletionPolicy = None
            self.UpdateReplacePolicy = None

    @property
    def is_taggable(self):
        """Is this resource taggable."""
        return hasattr(self.Properties, self.TAG_PROPERTY)

    def tag(self, tags=None, tag_derived_resources=True, **more_tags):
        """Apply tags to this resource, if they are supported.

        Existing tags with the same key will be overwritten.

        Parameters:
            tags (dict): Key-value pairs representing tags to apply to the
             resource.
            more_tags: Additional key-value pairs representing tags
             to apply to the resource, specified as keyword arguments.
             It is not defined which tag is actually applied in the event
             of duplicates.
            tag_derived_resources (bool): Whether to attempt to apply the
             same tags to resources which are derived from this one, using
             Cloud Formation (eg. EC2 instances started by an auto-scaling
             group).

        Returns:
            Whether tags are actually supported by this resource type.

        See Also:
            `AWS documentation on resource tagging
            <https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-resource-tags.html>`_
        """
        if not self.is_taggable:
            return False

        if tags is None:
            tags = {}

        # Add Tags to our horribly structured list
        for key, value in chain(tags.items(), more_tags.items()):
            # Ensure a tag with key and value exists, overwriting any existing
            # value with that key
            for tag_object in self._get_tag_list():
                if tag_object["Key"] == key:
                    # Overwrite the existing value
                    tag_object["Value"] = value
                    break
            else:
                # Key was not found, so append a new tag object
                self._get_tag_list().append({
                    "Key": key,
                    "Value": value,
                })

        return True

    def get_tag(self, key: str):
        """Get one of the tags currently set on this resource.

        Raises:
             AttributeError: If this resource doesn't support tags

        Returns:
            The tag's value, or else `None` if it is not set
        """
        # TODO tests
        if not self.is_taggable:
            raise AttributeError("Tags are not supported by {}".format(self.Type))

        # Search through existing tags looking for the tag
        for tag_object in self._get_tag_list():
            if tag_object["Key"] == key:
                return tag_object["Value"]

        # Tag was not found
        return None

    def _get_tag_list(self):
        """Get the internal list of all the tag objects on this resource,
        creating it if necessary
        """
        if not self.is_taggable:
            raise AttributeError("Tags are not supported by {}".format(self.Type))

        # Get or create the Tags property
        #
        # Note that Properties might be a `ResourceProperties` object, which
        # doesn't support `setdefault`
        taglist = self.Properties[self.TAG_PROPERTY]
        if taglist is not None:
            return taglist

        self.Properties[self.TAG_PROPERTY] = []
        return self.Properties[self.TAG_PROPERTY]

    @property
    def name(self):
        """The context-dependent name of this resource, if it is set.
        Usually a resource's name appears as the 'Name' tag.

        Raises:
             AttributeError: If this resource doesn't support name's

        Returns:
            The resource's name, or else `None` if it is not set
        """
        # The default implementation looks for the 'Name' tag
        return self.get_tag("Name")

    @name.setter
    def name(self, value: str):
        # The default implementation looks for the 'Name' tag
        if not self.is_taggable:
            raise AttributeError("Tags are not supported by {}".format(self.Type))

        self.tag(Name=value)


class ResourceProperties(AWSObject):
    """Base class for a Properties object on a Resource."""

    __slots__ = []

    # TODO put default tagging functionality in here, instead of on the resource?

    pass


class LogicalName(CustomYamlObject):
    """Represents the logical name of a resource in an exported stack.

    The logical name of a resource in a CloudFormation template is the
    internal variable name for that resource, which is unique within
    that template.

    Sometimes, when describing infrastructure with Flying Circus, you have a
    reference to the Flying Circus object but CloudFormation requires the
    template's logical name for the resource. This allows you to dereference
    the object, rather than carrying around hard-coded names (yay!)

    The name is exported to CloudFormation as a string, but internally it's an
    object that is dynamically resolved to the correct logical name on export.
    """

    # TODO Find a better module for this to live in
    # TODO consider subclassing from _Function
    # TODO also (primarily) be able to create these Name's from a method on the referenced object

    def __init__(self, resource: Resource):
        """Create a reference to the logical name of a Resource

        :param data: The Python object that is being referenced.
        """
        self._resource = resource

    def as_yaml_node(self, dumper):
        name = dumper.cfn_stack.get_logical_name(self._resource, resources_only=True)  # Pass error through
        return dumper.represent_str(name)


@attrs(**ATTRSCONFIG)
class Output(AWSObject):
    """Represents a CloudFormation Output.

    https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/outputs-section-structure.html
    """

    # TODO Export is a single-entry dictionary. Should we flatten it or do something nicer?

    Description: str = attrib(default=None)
    Export: Dict[str, Any] = attrib(factory=dict)
    Value: Any = attrib(default=None)


# TODO new `reflow` function that cleans a long (multi-) line string from IDE padding, and marks it as as suitable for PyYAML flow style

def dedent(st):
    """Remove unwanted whitespace from a multi-line string intended for output.

    This is perfect for text embedded inside indented Python code.
    """
    # TODO needs test cases
    # TODO different python module

    lines = st.split('\n')

    # Remove a leading line that contains only a newline character
    if lines and not lines[0]:
        lines = lines[1:]

    # Remove python-style leading indentation on each line
    cleaned_lines = textwrap.dedent('\n'.join(lines))

    return cleaned_lines



"""General-use classes to interact with the CodeBuild service through CloudFormation.

See Also:
    `AWS developer guide for CodeBuild
    <https://docs.aws.amazon.com/codebuild/latest/userguide/welcome.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import codebuild as _raw

# noinspection PyUnresolvedReferences
from .._raw.codebuild import *



"""Raw representations of every data type in the AWS DAX service.

See Also:
    `AWS developer guide for DAX
    <https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/DAX.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Cluster",
    "ClusterProperties",
    "ParameterGroup",
    "ParameterGroupProperties",
    "SubnetGroup",
    "SubnetGroupProperties",
]


@attrs(**ATTRSCONFIG)
class ClusterProperties(ResourceProperties):
    AvailabilityZones = attrib(default=None)
    ClusterName = attrib(default=None)
    Description = attrib(default=None)
    IAMRoleARN = attrib(default=None)
    NodeType = attrib(default=None)
    NotificationTopicARN = attrib(default=None)
    ParameterGroupName = attrib(default=None)
    PreferredMaintenanceWindow = attrib(default=None)
    ReplicationFactor = attrib(default=None)
    SecurityGroupIds = attrib(default=None)
    SSESpecification = attrib(default=None)
    SubnetGroupName = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Cluster(Resource):
    """A Cluster for DAX.

    See Also:
        `AWS Cloud Formation documentation for Cluster
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-cluster.html>`_
    """

    RESOURCE_TYPE = "AWS::DAX::Cluster"

    Properties: ClusterProperties = attrib(
        factory=ClusterProperties,
        converter=create_object_converter(ClusterProperties),
    )


@attrs(**ATTRSCONFIG)
class ParameterGroupProperties(ResourceProperties):
    Description = attrib(default=None)
    ParameterGroupName = attrib(default=None)
    ParameterNameValues = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ParameterGroup(Resource):
    """A Parameter Group for DAX.

    See Also:
        `AWS Cloud Formation documentation for ParameterGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-parametergroup.html>`_
    """

    RESOURCE_TYPE = "AWS::DAX::ParameterGroup"

    Properties: ParameterGroupProperties = attrib(
        factory=ParameterGroupProperties,
        converter=create_object_converter(ParameterGroupProperties),
    )


@attrs(**ATTRSCONFIG)
class SubnetGroupProperties(ResourceProperties):
    Description = attrib(default=None)
    SubnetGroupName = attrib(default=None)
    SubnetIds = attrib(default=None)


@attrs(**ATTRSCONFIG)
class SubnetGroup(Resource):
    """A Subnet Group for DAX.

    See Also:
        `AWS Cloud Formation documentation for SubnetGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-subnetgroup.html>`_
    """

    RESOURCE_TYPE = "AWS::DAX::SubnetGroup"

    Properties: SubnetGroupProperties = attrib(
        factory=SubnetGroupProperties,
        converter=create_object_converter(SubnetGroupProperties),
    )



"""Raw representations of every data type in the AWS KinesisAnalytics service.

See Also:
    `AWS developer guide for KinesisAnalytics
    <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/what-is.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Application",
    "ApplicationProperties",
    "ApplicationOutput",
    "ApplicationOutputProperties",
    "ApplicationReferenceDataSource",
    "ApplicationReferenceDataSourceProperties",
]


@attrs(**ATTRSCONFIG)
class ApplicationProperties(ResourceProperties):
    ApplicationCode = attrib(default=None)
    ApplicationDescription = attrib(default=None)
    ApplicationName = attrib(default=None)
    Inputs = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Application(Resource):
    """A Application for KinesisAnalytics.

    See Also:
        `AWS Cloud Formation documentation for Application
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-application.html>`_
    """

    RESOURCE_TYPE = "AWS::KinesisAnalytics::Application"

    Properties: ApplicationProperties = attrib(
        factory=ApplicationProperties,
        converter=create_object_converter(ApplicationProperties),
    )


@attrs(**ATTRSCONFIG)
class ApplicationOutputProperties(ResourceProperties):
    ApplicationName = attrib(default=None)
    Output = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ApplicationOutput(Resource):
    """A Application Output for KinesisAnalytics.

    See Also:
        `AWS Cloud Formation documentation for ApplicationOutput
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-applicationoutput.html>`_
    """

    RESOURCE_TYPE = "AWS::KinesisAnalytics::ApplicationOutput"

    Properties: ApplicationOutputProperties = attrib(
        factory=ApplicationOutputProperties,
        converter=create_object_converter(ApplicationOutputProperties),
    )


@attrs(**ATTRSCONFIG)
class ApplicationReferenceDataSourceProperties(ResourceProperties):
    ApplicationName = attrib(default=None)
    ReferenceDataSource = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ApplicationReferenceDataSource(Resource):
    """A Application Reference Data Source for KinesisAnalytics.

    See Also:
        `AWS Cloud Formation documentation for ApplicationReferenceDataSource
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-applicationreferencedatasource.html>`_
    """

    RESOURCE_TYPE = "AWS::KinesisAnalytics::ApplicationReferenceDataSource"

    Properties: ApplicationReferenceDataSourceProperties = attrib(
        factory=ApplicationReferenceDataSourceProperties,
        converter=create_object_converter(ApplicationReferenceDataSourceProperties),
    )



"""Raw representations of every data type in the AWS Route53 service.

See Also:
    `AWS developer guide for Route53
    <https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "HealthCheck",
    "HealthCheckProperties",
    "HostedZone",
    "HostedZoneProperties",
    "RecordSet",
    "RecordSetProperties",
    "RecordSetGroup",
    "RecordSetGroupProperties",
]


@attrs(**ATTRSCONFIG)
class HealthCheckProperties(ResourceProperties):
    HealthCheckConfig = attrib(default=None)
    HealthCheckTags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class HealthCheck(Resource):
    """A Health Check for Route53.

    See Also:
        `AWS Cloud Formation documentation for HealthCheck
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-healthcheck.html>`_
    """

    RESOURCE_TYPE = "AWS::Route53::HealthCheck"

    Properties: HealthCheckProperties = attrib(
        factory=HealthCheckProperties,
        converter=create_object_converter(HealthCheckProperties),
    )


@attrs(**ATTRSCONFIG)
class HostedZoneProperties(ResourceProperties):
    HostedZoneConfig = attrib(default=None)
    HostedZoneTags = attrib(default=None)
    Name = attrib(default=None)
    QueryLoggingConfig = attrib(default=None)
    VPCs = attrib(default=None)


@attrs(**ATTRSCONFIG)
class HostedZone(Resource):
    """A Hosted Zone for Route53.

    See Also:
        `AWS Cloud Formation documentation for HostedZone
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-hostedzone.html>`_
    """

    RESOURCE_TYPE = "AWS::Route53::HostedZone"

    Properties: HostedZoneProperties = attrib(
        factory=HostedZoneProperties,
        converter=create_object_converter(HostedZoneProperties),
    )


@attrs(**ATTRSCONFIG)
class RecordSetProperties(ResourceProperties):
    AliasTarget = attrib(default=None)
    Comment = attrib(default=None)
    Failover = attrib(default=None)
    GeoLocation = attrib(default=None)
    HealthCheckId = attrib(default=None)
    HostedZoneId = attrib(default=None)
    HostedZoneName = attrib(default=None)
    MultiValueAnswer = attrib(default=None)
    Name = attrib(default=None)
    Region = attrib(default=None)
    ResourceRecords = attrib(default=None)
    SetIdentifier = attrib(default=None)
    TTL = attrib(default=None)
    Type = attrib(default=None)
    Weight = attrib(default=None)


@attrs(**ATTRSCONFIG)
class RecordSet(Resource):
    """A Record Set for Route53.

    See Also:
        `AWS Cloud Formation documentation for RecordSet
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html>`_
    """

    RESOURCE_TYPE = "AWS::Route53::RecordSet"

    Properties: RecordSetProperties = attrib(
        factory=RecordSetProperties,
        converter=create_object_converter(RecordSetProperties),
    )


@attrs(**ATTRSCONFIG)
class RecordSetGroupProperties(ResourceProperties):
    Comment = attrib(default=None)
    HostedZoneId = attrib(default=None)
    HostedZoneName = attrib(default=None)
    RecordSets = attrib(default=None)


@attrs(**ATTRSCONFIG)
class RecordSetGroup(Resource):
    """A Record Set Group for Route53.

    See Also:
        `AWS Cloud Formation documentation for RecordSetGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-recordsetgroup.html>`_
    """

    RESOURCE_TYPE = "AWS::Route53::RecordSetGroup"

    Properties: RecordSetGroupProperties = attrib(
        factory=RecordSetGroupProperties,
        converter=create_object_converter(RecordSetGroupProperties),
    )



"""Raw representations of every data type in the AWS DLM service.

See Also:
    `AWS developer guide for DLM
    <https://docs.aws.amazon.com/dlm/latest/APIReference/Welcome.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "LifecyclePolicy",
    "LifecyclePolicyProperties",
]


@attrs(**ATTRSCONFIG)
class LifecyclePolicyProperties(ResourceProperties):
    Description = attrib(default=None)
    ExecutionRoleArn = attrib(default=None)
    PolicyDetails = attrib(default=None)
    State = attrib(default=None)


@attrs(**ATTRSCONFIG)
class LifecyclePolicy(Resource):
    """A Lifecycle Policy for DLM.

    See Also:
        `AWS Cloud Formation documentation for LifecyclePolicy
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dlm-lifecyclepolicy.html>`_
    """

    RESOURCE_TYPE = "AWS::DLM::LifecyclePolicy"

    Properties: LifecyclePolicyProperties = attrib(
        factory=LifecyclePolicyProperties,
        converter=create_object_converter(LifecyclePolicyProperties),
    )



"""General-use classes to interact with the Inspector service through CloudFormation.

See Also:
    `AWS developer guide for Inspector
    <https://docs.aws.amazon.com/inspector/latest/userguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import inspector as _raw

# noinspection PyUnresolvedReferences
from .._raw.inspector import *



"""General-use classes to interact with the SQS service through CloudFormation.

See Also:
    `AWS developer guide for SQS
    <https://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/welcome.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import sqs as _raw

# noinspection PyUnresolvedReferences
from .._raw.sqs import *



"""Raw representations of every data type in the AWS AmazonMQ service.

See Also:
    `AWS developer guide for AmazonMQ
    <https://docs.aws.amazon.com/amazon-mq/latest/developer-guide/welcome.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Broker",
    "BrokerProperties",
    "Configuration",
    "ConfigurationProperties",
    "ConfigurationAssociation",
    "ConfigurationAssociationProperties",
]


@attrs(**ATTRSCONFIG)
class BrokerProperties(ResourceProperties):
    AutoMinorVersionUpgrade = attrib(default=None)
    BrokerName = attrib(default=None)
    Configuration = attrib(default=None)
    DeploymentMode = attrib(default=None)
    EngineType = attrib(default=None)
    EngineVersion = attrib(default=None)
    HostInstanceType = attrib(default=None)
    Logs = attrib(default=None)
    MaintenanceWindowStartTime = attrib(default=None)
    PubliclyAccessible = attrib(default=None)
    SecurityGroups = attrib(default=None)
    SubnetIds = attrib(default=None)
    Tags = attrib(default=None)
    Users = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Broker(Resource):
    """A Broker for AmazonMQ.

    See Also:
        `AWS Cloud Formation documentation for Broker
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-amazonmq-broker.html>`_
    """

    RESOURCE_TYPE = "AWS::AmazonMQ::Broker"

    Properties: BrokerProperties = attrib(
        factory=BrokerProperties,
        converter=create_object_converter(BrokerProperties),
    )


@attrs(**ATTRSCONFIG)
class ConfigurationProperties(ResourceProperties):
    Data = attrib(default=None)
    Description = attrib(default=None)
    EngineType = attrib(default=None)
    EngineVersion = attrib(default=None)
    Name = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Configuration(Resource):
    """A Configuration for AmazonMQ.

    See Also:
        `AWS Cloud Formation documentation for Configuration
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-amazonmq-configuration.html>`_
    """

    RESOURCE_TYPE = "AWS::AmazonMQ::Configuration"

    Properties: ConfigurationProperties = attrib(
        factory=ConfigurationProperties,
        converter=create_object_converter(ConfigurationProperties),
    )


@attrs(**ATTRSCONFIG)
class ConfigurationAssociationProperties(ResourceProperties):
    Broker = attrib(default=None)
    Configuration = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ConfigurationAssociation(Resource):
    """A Configuration Association for AmazonMQ.

    See Also:
        `AWS Cloud Formation documentation for ConfigurationAssociation
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-amazonmq-configurationassociation.html>`_
    """

    RESOURCE_TYPE = "AWS::AmazonMQ::ConfigurationAssociation"

    Properties: ConfigurationAssociationProperties = attrib(
        factory=ConfigurationAssociationProperties,
        converter=create_object_converter(ConfigurationAssociationProperties),
    )



"""General-use classes to interact with the Config service through CloudFormation.

See Also:
    `AWS developer guide for Config
    <https://docs.aws.amazon.com/config/latest/developerguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import config as _raw

# noinspection PyUnresolvedReferences
from .._raw.config import *



"""Raw representations of every data type in the AWS ApiGateway service.

See Also:
    `AWS developer guide for ApiGateway
    <https://docs.aws.amazon.com/apigateway/latest/developerguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Account",
    "AccountProperties",
    "ApiKey",
    "ApiKeyProperties",
    "Authorizer",
    "AuthorizerProperties",
    "BasePathMapping",
    "BasePathMappingProperties",
    "ClientCertificate",
    "ClientCertificateProperties",
    "Deployment",
    "DeploymentProperties",
    "DocumentationPart",
    "DocumentationPartProperties",
    "DocumentationVersion",
    "DocumentationVersionProperties",
    "DomainName",
    "DomainNameProperties",
    "GatewayResponse",
    "GatewayResponseProperties",
    "Method",
    "MethodProperties",
    "Model",
    "ModelProperties",
    "RequestValidator",
    "RequestValidatorProperties",
    "Resource",
    "ResourceProperties",
    "RestApi",
    "RestApiProperties",
    "Stage",
    "StageProperties",
    "UsagePlan",
    "UsagePlanProperties",
    "UsagePlanKey",
    "UsagePlanKeyProperties",
    "VpcLink",
    "VpcLinkProperties",
]


@attrs(**ATTRSCONFIG)
class AccountProperties(ResourceProperties):
    CloudWatchRoleArn = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Account(Resource):
    """A Account for ApiGateway.

    See Also:
        `AWS Cloud Formation documentation for Account
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-account.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGateway::Account"

    Properties: AccountProperties = attrib(
        factory=AccountProperties,
        converter=create_object_converter(AccountProperties),
    )


@attrs(**ATTRSCONFIG)
class ApiKeyProperties(ResourceProperties):
    CustomerId = attrib(default=None)
    Description = attrib(default=None)
    Enabled = attrib(default=None)
    GenerateDistinctId = attrib(default=None)
    Name = attrib(default=None)
    StageKeys = attrib(default=None)
    Value = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ApiKey(Resource):
    """A Api Key for ApiGateway.

    See Also:
        `AWS Cloud Formation documentation for ApiKey
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-apikey.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGateway::ApiKey"

    Properties: ApiKeyProperties = attrib(
        factory=ApiKeyProperties,
        converter=create_object_converter(ApiKeyProperties),
    )


@attrs(**ATTRSCONFIG)
class AuthorizerProperties(ResourceProperties):
    AuthorizerCredentials = attrib(default=None)
    AuthorizerResultTtlInSeconds = attrib(default=None)
    AuthorizerUri = attrib(default=None)
    AuthType = attrib(default=None)
    IdentitySource = attrib(default=None)
    IdentityValidationExpression = attrib(default=None)
    Name = attrib(default=None)
    ProviderARNs = attrib(default=None)
    RestApiId = attrib(default=None)
    Type = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Authorizer(Resource):
    """A Authorizer for ApiGateway.

    See Also:
        `AWS Cloud Formation documentation for Authorizer
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGateway::Authorizer"

    Properties: AuthorizerProperties = attrib(
        factory=AuthorizerProperties,
        converter=create_object_converter(AuthorizerProperties),
    )


@attrs(**ATTRSCONFIG)
class BasePathMappingProperties(ResourceProperties):
    BasePath = attrib(default=None)
    DomainName = attrib(default=None)
    RestApiId = attrib(default=None)
    Stage = attrib(default=None)


@attrs(**ATTRSCONFIG)
class BasePathMapping(Resource):
    """A Base Path Mapping for ApiGateway.

    See Also:
        `AWS Cloud Formation documentation for BasePathMapping
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-basepathmapping.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGateway::BasePathMapping"

    Properties: BasePathMappingProperties = attrib(
        factory=BasePathMappingProperties,
        converter=create_object_converter(BasePathMappingProperties),
    )


@attrs(**ATTRSCONFIG)
class ClientCertificateProperties(ResourceProperties):
    Description = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ClientCertificate(Resource):
    """A Client Certificate for ApiGateway.

    See Also:
        `AWS Cloud Formation documentation for ClientCertificate
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-clientcertificate.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGateway::ClientCertificate"

    Properties: ClientCertificateProperties = attrib(
        factory=ClientCertificateProperties,
        converter=create_object_converter(ClientCertificateProperties),
    )


@attrs(**ATTRSCONFIG)
class DeploymentProperties(ResourceProperties):
    DeploymentCanarySettings = attrib(default=None)
    Description = attrib(default=None)
    RestApiId = attrib(default=None)
    StageDescription = attrib(default=None)
    StageName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Deployment(Resource):
    """A Deployment for ApiGateway.

    See Also:
        `AWS Cloud Formation documentation for Deployment
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-deployment.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGateway::Deployment"

    Properties: DeploymentProperties = attrib(
        factory=DeploymentProperties,
        converter=create_object_converter(DeploymentProperties),
    )


@attrs(**ATTRSCONFIG)
class DocumentationPartProperties(ResourceProperties):
    Location = attrib(default=None)
    Properties = attrib(default=None)
    RestApiId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DocumentationPart(Resource):
    """A Documentation Part for ApiGateway.

    See Also:
        `AWS Cloud Formation documentation for DocumentationPart
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-documentationpart.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGateway::DocumentationPart"

    Properties: DocumentationPartProperties = attrib(
        factory=DocumentationPartProperties,
        converter=create_object_converter(DocumentationPartProperties),
    )


@attrs(**ATTRSCONFIG)
class DocumentationVersionProperties(ResourceProperties):
    Description = attrib(default=None)
    DocumentationVersion = attrib(default=None)
    RestApiId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DocumentationVersion(Resource):
    """A Documentation Version for ApiGateway.

    See Also:
        `AWS Cloud Formation documentation for DocumentationVersion
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-documentationversion.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGateway::DocumentationVersion"

    Properties: DocumentationVersionProperties = attrib(
        factory=DocumentationVersionProperties,
        converter=create_object_converter(DocumentationVersionProperties),
    )


@attrs(**ATTRSCONFIG)
class DomainNameProperties(ResourceProperties):
    CertificateArn = attrib(default=None)
    DomainName = attrib(default=None)
    EndpointConfiguration = attrib(default=None)
    RegionalCertificateArn = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DomainName(Resource):
    """A Domain Name for ApiGateway.

    See Also:
        `AWS Cloud Formation documentation for DomainName
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-domainname.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGateway::DomainName"

    Properties: DomainNameProperties = attrib(
        factory=DomainNameProperties,
        converter=create_object_converter(DomainNameProperties),
    )


@attrs(**ATTRSCONFIG)
class GatewayResponseProperties(ResourceProperties):
    ResponseParameters = attrib(default=None)
    ResponseTemplates = attrib(default=None)
    ResponseType = attrib(default=None)
    RestApiId = attrib(default=None)
    StatusCode = attrib(default=None)


@attrs(**ATTRSCONFIG)
class GatewayResponse(Resource):
    """A Gateway Response for ApiGateway.

    See Also:
        `AWS Cloud Formation documentation for GatewayResponse
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-gatewayresponse.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGateway::GatewayResponse"

    Properties: GatewayResponseProperties = attrib(
        factory=GatewayResponseProperties,
        converter=create_object_converter(GatewayResponseProperties),
    )


@attrs(**ATTRSCONFIG)
class MethodProperties(ResourceProperties):
    ApiKeyRequired = attrib(default=None)
    AuthorizationScopes = attrib(default=None)
    AuthorizationType = attrib(default=None)
    AuthorizerId = attrib(default=None)
    HttpMethod = attrib(default=None)
    Integration = attrib(default=None)
    MethodResponses = attrib(default=None)
    OperationName = attrib(default=None)
    RequestModels = attrib(default=None)
    RequestParameters = attrib(default=None)
    RequestValidatorId = attrib(default=None)
    ResourceId = attrib(default=None)
    RestApiId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Method(Resource):
    """A Method for ApiGateway.

    See Also:
        `AWS Cloud Formation documentation for Method
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGateway::Method"

    Properties: MethodProperties = attrib(
        factory=MethodProperties,
        converter=create_object_converter(MethodProperties),
    )


@attrs(**ATTRSCONFIG)
class ModelProperties(ResourceProperties):
    ContentType = attrib(default=None)
    Description = attrib(default=None)
    Name = attrib(default=None)
    RestApiId = attrib(default=None)
    Schema = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Model(Resource):
    """A Model for ApiGateway.

    See Also:
        `AWS Cloud Formation documentation for Model
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-model.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGateway::Model"

    Properties: ModelProperties = attrib(
        factory=ModelProperties,
        converter=create_object_converter(ModelProperties),
    )


@attrs(**ATTRSCONFIG)
class RequestValidatorProperties(ResourceProperties):
    Name = attrib(default=None)
    RestApiId = attrib(default=None)
    ValidateRequestBody = attrib(default=None)
    ValidateRequestParameters = attrib(default=None)


@attrs(**ATTRSCONFIG)
class RequestValidator(Resource):
    """A Request Validator for ApiGateway.

    See Also:
        `AWS Cloud Formation documentation for RequestValidator
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-requestvalidator.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGateway::RequestValidator"

    Properties: RequestValidatorProperties = attrib(
        factory=RequestValidatorProperties,
        converter=create_object_converter(RequestValidatorProperties),
    )


@attrs(**ATTRSCONFIG)
class ResourceProperties(ResourceProperties):
    ParentId = attrib(default=None)
    PathPart = attrib(default=None)
    RestApiId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Resource(Resource):
    """A Resource for ApiGateway.

    See Also:
        `AWS Cloud Formation documentation for Resource
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-resource.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGateway::Resource"

    Properties: ResourceProperties = attrib(
        factory=ResourceProperties,
        converter=create_object_converter(ResourceProperties),
    )


@attrs(**ATTRSCONFIG)
class RestApiProperties(ResourceProperties):
    ApiKeySourceType = attrib(default=None)
    BinaryMediaTypes = attrib(default=None)
    Body = attrib(default=None)
    BodyS3Location = attrib(default=None)
    CloneFrom = attrib(default=None)
    Description = attrib(default=None)
    EndpointConfiguration = attrib(default=None)
    FailOnWarnings = attrib(default=None)
    MinimumCompressionSize = attrib(default=None)
    Name = attrib(default=None)
    Parameters = attrib(default=None)
    Policy = attrib(default=None)


@attrs(**ATTRSCONFIG)
class RestApi(Resource):
    """A Rest Api for ApiGateway.

    See Also:
        `AWS Cloud Formation documentation for RestApi
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-restapi.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGateway::RestApi"

    Properties: RestApiProperties = attrib(
        factory=RestApiProperties,
        converter=create_object_converter(RestApiProperties),
    )


@attrs(**ATTRSCONFIG)
class StageProperties(ResourceProperties):
    AccessLogSetting = attrib(default=None)
    CacheClusterEnabled = attrib(default=None)
    CacheClusterSize = attrib(default=None)
    CanarySetting = attrib(default=None)
    ClientCertificateId = attrib(default=None)
    DeploymentId = attrib(default=None)
    Description = attrib(default=None)
    DocumentationVersion = attrib(default=None)
    MethodSettings = attrib(default=None)
    RestApiId = attrib(default=None)
    StageName = attrib(default=None)
    Tags = attrib(default=None)
    TracingEnabled = attrib(default=None)
    Variables = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Stage(Resource):
    """A Stage for ApiGateway.

    See Also:
        `AWS Cloud Formation documentation for Stage
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-stage.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGateway::Stage"

    Properties: StageProperties = attrib(
        factory=StageProperties,
        converter=create_object_converter(StageProperties),
    )


@attrs(**ATTRSCONFIG)
class UsagePlanProperties(ResourceProperties):
    ApiStages = attrib(default=None)
    Description = attrib(default=None)
    Quota = attrib(default=None)
    Throttle = attrib(default=None)
    UsagePlanName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class UsagePlan(Resource):
    """A Usage Plan for ApiGateway.

    See Also:
        `AWS Cloud Formation documentation for UsagePlan
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-usageplan.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGateway::UsagePlan"

    Properties: UsagePlanProperties = attrib(
        factory=UsagePlanProperties,
        converter=create_object_converter(UsagePlanProperties),
    )


@attrs(**ATTRSCONFIG)
class UsagePlanKeyProperties(ResourceProperties):
    KeyId = attrib(default=None)
    KeyType = attrib(default=None)
    UsagePlanId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class UsagePlanKey(Resource):
    """A Usage Plan Key for ApiGateway.

    See Also:
        `AWS Cloud Formation documentation for UsagePlanKey
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-usageplankey.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGateway::UsagePlanKey"

    Properties: UsagePlanKeyProperties = attrib(
        factory=UsagePlanKeyProperties,
        converter=create_object_converter(UsagePlanKeyProperties),
    )


@attrs(**ATTRSCONFIG)
class VpcLinkProperties(ResourceProperties):
    Description = attrib(default=None)
    Name = attrib(default=None)
    TargetArns = attrib(default=None)


@attrs(**ATTRSCONFIG)
class VpcLink(Resource):
    """A Vpc Link for ApiGateway.

    See Also:
        `AWS Cloud Formation documentation for VpcLink
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-vpclink.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGateway::VpcLink"

    Properties: VpcLinkProperties = attrib(
        factory=VpcLinkProperties,
        converter=create_object_converter(VpcLinkProperties),
    )



"""Raw representations of every data type in the AWS SecretsManager service.

See Also:
    `AWS developer guide for SecretsManager
    <https://docs.aws.amazon.com/secretsmanager/latest/userguide/intro.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "ResourcePolicy",
    "ResourcePolicyProperties",
    "RotationSchedule",
    "RotationScheduleProperties",
    "Secret",
    "SecretProperties",
    "SecretTargetAttachment",
    "SecretTargetAttachmentProperties",
]


@attrs(**ATTRSCONFIG)
class ResourcePolicyProperties(ResourceProperties):
    ResourcePolicy = attrib(default=None)
    SecretId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ResourcePolicy(Resource):
    """A Resource Policy for SecretsManager.

    See Also:
        `AWS Cloud Formation documentation for ResourcePolicy
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-secretsmanager-resourcepolicy.html>`_
    """

    RESOURCE_TYPE = "AWS::SecretsManager::ResourcePolicy"

    Properties: ResourcePolicyProperties = attrib(
        factory=ResourcePolicyProperties,
        converter=create_object_converter(ResourcePolicyProperties),
    )


@attrs(**ATTRSCONFIG)
class RotationScheduleProperties(ResourceProperties):
    RotationLambdaARN = attrib(default=None)
    RotationRules = attrib(default=None)
    SecretId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class RotationSchedule(Resource):
    """A Rotation Schedule for SecretsManager.

    See Also:
        `AWS Cloud Formation documentation for RotationSchedule
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-secretsmanager-rotationschedule.html>`_
    """

    RESOURCE_TYPE = "AWS::SecretsManager::RotationSchedule"

    Properties: RotationScheduleProperties = attrib(
        factory=RotationScheduleProperties,
        converter=create_object_converter(RotationScheduleProperties),
    )


@attrs(**ATTRSCONFIG)
class SecretProperties(ResourceProperties):
    Description = attrib(default=None)
    GenerateSecretString = attrib(default=None)
    KmsKeyId = attrib(default=None)
    Name = attrib(default=None)
    SecretString = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Secret(Resource):
    """A Secret for SecretsManager.

    See Also:
        `AWS Cloud Formation documentation for Secret
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-secretsmanager-secret.html>`_
    """

    RESOURCE_TYPE = "AWS::SecretsManager::Secret"

    Properties: SecretProperties = attrib(
        factory=SecretProperties,
        converter=create_object_converter(SecretProperties),
    )


@attrs(**ATTRSCONFIG)
class SecretTargetAttachmentProperties(ResourceProperties):
    SecretId = attrib(default=None)
    TargetId = attrib(default=None)
    TargetType = attrib(default=None)


@attrs(**ATTRSCONFIG)
class SecretTargetAttachment(Resource):
    """A Secret Target Attachment for SecretsManager.

    See Also:
        `AWS Cloud Formation documentation for SecretTargetAttachment
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-secretsmanager-secrettargetattachment.html>`_
    """

    RESOURCE_TYPE = "AWS::SecretsManager::SecretTargetAttachment"

    Properties: SecretTargetAttachmentProperties = attrib(
        factory=SecretTargetAttachmentProperties,
        converter=create_object_converter(SecretTargetAttachmentProperties),
    )



"""General-use classes to interact with the DAX service through CloudFormation.

See Also:
    `AWS developer guide for DAX
    <https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/DAX.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import dax as _raw

# noinspection PyUnresolvedReferences
from .._raw.dax import *



"""Raw representations of every data type in the AWS ServiceCatalog service.

See Also:
    `AWS developer guide for ServiceCatalog
    <https://docs.aws.amazon.com/servicecatalog/latest/dg/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "AcceptedPortfolioShare",
    "AcceptedPortfolioShareProperties",
    "CloudFormationProduct",
    "CloudFormationProductProperties",
    "CloudFormationProvisionedProduct",
    "CloudFormationProvisionedProductProperties",
    "LaunchNotificationConstraint",
    "LaunchNotificationConstraintProperties",
    "LaunchRoleConstraint",
    "LaunchRoleConstraintProperties",
    "LaunchTemplateConstraint",
    "LaunchTemplateConstraintProperties",
    "Portfolio",
    "PortfolioProperties",
    "PortfolioPrincipalAssociation",
    "PortfolioPrincipalAssociationProperties",
    "PortfolioProductAssociation",
    "PortfolioProductAssociationProperties",
    "PortfolioShare",
    "PortfolioShareProperties",
    "TagOption",
    "TagOptionProperties",
    "TagOptionAssociation",
    "TagOptionAssociationProperties",
]


@attrs(**ATTRSCONFIG)
class AcceptedPortfolioShareProperties(ResourceProperties):
    AcceptLanguage = attrib(default=None)
    PortfolioId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class AcceptedPortfolioShare(Resource):
    """A Accepted Portfolio Share for ServiceCatalog.

    See Also:
        `AWS Cloud Formation documentation for AcceptedPortfolioShare
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-acceptedportfolioshare.html>`_
    """

    RESOURCE_TYPE = "AWS::ServiceCatalog::AcceptedPortfolioShare"

    Properties: AcceptedPortfolioShareProperties = attrib(
        factory=AcceptedPortfolioShareProperties,
        converter=create_object_converter(AcceptedPortfolioShareProperties),
    )


@attrs(**ATTRSCONFIG)
class CloudFormationProductProperties(ResourceProperties):
    AcceptLanguage = attrib(default=None)
    Description = attrib(default=None)
    Distributor = attrib(default=None)
    Name = attrib(default=None)
    Owner = attrib(default=None)
    ProvisioningArtifactParameters = attrib(default=None)
    SupportDescription = attrib(default=None)
    SupportEmail = attrib(default=None)
    SupportUrl = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class CloudFormationProduct(Resource):
    """A Cloud Formation Product for ServiceCatalog.

    See Also:
        `AWS Cloud Formation documentation for CloudFormationProduct
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-cloudformationproduct.html>`_
    """

    RESOURCE_TYPE = "AWS::ServiceCatalog::CloudFormationProduct"

    Properties: CloudFormationProductProperties = attrib(
        factory=CloudFormationProductProperties,
        converter=create_object_converter(CloudFormationProductProperties),
    )


@attrs(**ATTRSCONFIG)
class CloudFormationProvisionedProductProperties(ResourceProperties):
    AcceptLanguage = attrib(default=None)
    NotificationArns = attrib(default=None)
    PathId = attrib(default=None)
    ProductId = attrib(default=None)
    ProductName = attrib(default=None)
    ProvisionedProductName = attrib(default=None)
    ProvisioningArtifactId = attrib(default=None)
    ProvisioningArtifactName = attrib(default=None)
    ProvisioningParameters = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class CloudFormationProvisionedProduct(Resource):
    """A Cloud Formation Provisioned Product for ServiceCatalog.

    See Also:
        `AWS Cloud Formation documentation for CloudFormationProvisionedProduct
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-cloudformationprovisionedproduct.html>`_
    """

    RESOURCE_TYPE = "AWS::ServiceCatalog::CloudFormationProvisionedProduct"

    Properties: CloudFormationProvisionedProductProperties = attrib(
        factory=CloudFormationProvisionedProductProperties,
        converter=create_object_converter(CloudFormationProvisionedProductProperties),
    )


@attrs(**ATTRSCONFIG)
class LaunchNotificationConstraintProperties(ResourceProperties):
    AcceptLanguage = attrib(default=None)
    Description = attrib(default=None)
    NotificationArns = attrib(default=None)
    PortfolioId = attrib(default=None)
    ProductId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class LaunchNotificationConstraint(Resource):
    """A Launch Notification Constraint for ServiceCatalog.

    See Also:
        `AWS Cloud Formation documentation for LaunchNotificationConstraint
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-launchnotificationconstraint.html>`_
    """

    RESOURCE_TYPE = "AWS::ServiceCatalog::LaunchNotificationConstraint"

    Properties: LaunchNotificationConstraintProperties = attrib(
        factory=LaunchNotificationConstraintProperties,
        converter=create_object_converter(LaunchNotificationConstraintProperties),
    )


@attrs(**ATTRSCONFIG)
class LaunchRoleConstraintProperties(ResourceProperties):
    AcceptLanguage = attrib(default=None)
    Description = attrib(default=None)
    PortfolioId = attrib(default=None)
    ProductId = attrib(default=None)
    RoleArn = attrib(default=None)


@attrs(**ATTRSCONFIG)
class LaunchRoleConstraint(Resource):
    """A Launch Role Constraint for ServiceCatalog.

    See Also:
        `AWS Cloud Formation documentation for LaunchRoleConstraint
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-launchroleconstraint.html>`_
    """

    RESOURCE_TYPE = "AWS::ServiceCatalog::LaunchRoleConstraint"

    Properties: LaunchRoleConstraintProperties = attrib(
        factory=LaunchRoleConstraintProperties,
        converter=create_object_converter(LaunchRoleConstraintProperties),
    )


@attrs(**ATTRSCONFIG)
class LaunchTemplateConstraintProperties(ResourceProperties):
    AcceptLanguage = attrib(default=None)
    Description = attrib(default=None)
    PortfolioId = attrib(default=None)
    ProductId = attrib(default=None)
    Rules = attrib(default=None)


@attrs(**ATTRSCONFIG)
class LaunchTemplateConstraint(Resource):
    """A Launch Template Constraint for ServiceCatalog.

    See Also:
        `AWS Cloud Formation documentation for LaunchTemplateConstraint
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-launchtemplateconstraint.html>`_
    """

    RESOURCE_TYPE = "AWS::ServiceCatalog::LaunchTemplateConstraint"

    Properties: LaunchTemplateConstraintProperties = attrib(
        factory=LaunchTemplateConstraintProperties,
        converter=create_object_converter(LaunchTemplateConstraintProperties),
    )


@attrs(**ATTRSCONFIG)
class PortfolioProperties(ResourceProperties):
    AcceptLanguage = attrib(default=None)
    Description = attrib(default=None)
    DisplayName = attrib(default=None)
    ProviderName = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Portfolio(Resource):
    """A Portfolio for ServiceCatalog.

    See Also:
        `AWS Cloud Formation documentation for Portfolio
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-portfolio.html>`_
    """

    RESOURCE_TYPE = "AWS::ServiceCatalog::Portfolio"

    Properties: PortfolioProperties = attrib(
        factory=PortfolioProperties,
        converter=create_object_converter(PortfolioProperties),
    )


@attrs(**ATTRSCONFIG)
class PortfolioPrincipalAssociationProperties(ResourceProperties):
    AcceptLanguage = attrib(default=None)
    PortfolioId = attrib(default=None)
    PrincipalARN = attrib(default=None)
    PrincipalType = attrib(default=None)


@attrs(**ATTRSCONFIG)
class PortfolioPrincipalAssociation(Resource):
    """A Portfolio Principal Association for ServiceCatalog.

    See Also:
        `AWS Cloud Formation documentation for PortfolioPrincipalAssociation
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-portfolioprincipalassociation.html>`_
    """

    RESOURCE_TYPE = "AWS::ServiceCatalog::PortfolioPrincipalAssociation"

    Properties: PortfolioPrincipalAssociationProperties = attrib(
        factory=PortfolioPrincipalAssociationProperties,
        converter=create_object_converter(PortfolioPrincipalAssociationProperties),
    )


@attrs(**ATTRSCONFIG)
class PortfolioProductAssociationProperties(ResourceProperties):
    AcceptLanguage = attrib(default=None)
    PortfolioId = attrib(default=None)
    ProductId = attrib(default=None)
    SourcePortfolioId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class PortfolioProductAssociation(Resource):
    """A Portfolio Product Association for ServiceCatalog.

    See Also:
        `AWS Cloud Formation documentation for PortfolioProductAssociation
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-portfolioproductassociation.html>`_
    """

    RESOURCE_TYPE = "AWS::ServiceCatalog::PortfolioProductAssociation"

    Properties: PortfolioProductAssociationProperties = attrib(
        factory=PortfolioProductAssociationProperties,
        converter=create_object_converter(PortfolioProductAssociationProperties),
    )


@attrs(**ATTRSCONFIG)
class PortfolioShareProperties(ResourceProperties):
    AcceptLanguage = attrib(default=None)
    AccountId = attrib(default=None)
    PortfolioId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class PortfolioShare(Resource):
    """A Portfolio Share for ServiceCatalog.

    See Also:
        `AWS Cloud Formation documentation for PortfolioShare
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-portfolioshare.html>`_
    """

    RESOURCE_TYPE = "AWS::ServiceCatalog::PortfolioShare"

    Properties: PortfolioShareProperties = attrib(
        factory=PortfolioShareProperties,
        converter=create_object_converter(PortfolioShareProperties),
    )


@attrs(**ATTRSCONFIG)
class TagOptionProperties(ResourceProperties):
    Active = attrib(default=None)
    Key = attrib(default=None)
    Value = attrib(default=None)


@attrs(**ATTRSCONFIG)
class TagOption(Resource):
    """A Tag Option for ServiceCatalog.

    See Also:
        `AWS Cloud Formation documentation for TagOption
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-tagoption.html>`_
    """

    RESOURCE_TYPE = "AWS::ServiceCatalog::TagOption"

    Properties: TagOptionProperties = attrib(
        factory=TagOptionProperties,
        converter=create_object_converter(TagOptionProperties),
    )


@attrs(**ATTRSCONFIG)
class TagOptionAssociationProperties(ResourceProperties):
    ResourceId = attrib(default=None)
    TagOptionId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class TagOptionAssociation(Resource):
    """A Tag Option Association for ServiceCatalog.

    See Also:
        `AWS Cloud Formation documentation for TagOptionAssociation
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-tagoptionassociation.html>`_
    """

    RESOURCE_TYPE = "AWS::ServiceCatalog::TagOptionAssociation"

    Properties: TagOptionAssociationProperties = attrib(
        factory=TagOptionAssociationProperties,
        converter=create_object_converter(TagOptionAssociationProperties),
    )



"""Raw representations of every data type in the AWS EMR service.

See Also:
    `AWS developer guide for EMR
    <https://docs.aws.amazon.com/dms/latest/userguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Cluster",
    "ClusterProperties",
    "InstanceFleetConfig",
    "InstanceFleetConfigProperties",
    "InstanceGroupConfig",
    "InstanceGroupConfigProperties",
    "SecurityConfiguration",
    "SecurityConfigurationProperties",
    "Step",
    "StepProperties",
]


@attrs(**ATTRSCONFIG)
class ClusterProperties(ResourceProperties):
    AdditionalInfo = attrib(default=None)
    Applications = attrib(default=None)
    AutoScalingRole = attrib(default=None)
    BootstrapActions = attrib(default=None)
    Configurations = attrib(default=None)
    CustomAmiId = attrib(default=None)
    EbsRootVolumeSize = attrib(default=None)
    Instances = attrib(default=None)
    JobFlowRole = attrib(default=None)
    KerberosAttributes = attrib(default=None)
    LogUri = attrib(default=None)
    Name = attrib(default=None)
    ReleaseLabel = attrib(default=None)
    ScaleDownBehavior = attrib(default=None)
    SecurityConfiguration = attrib(default=None)
    ServiceRole = attrib(default=None)
    Steps = attrib(default=None)
    Tags = attrib(default=None)
    VisibleToAllUsers = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Cluster(Resource):
    """A Cluster for EMR.

    See Also:
        `AWS Cloud Formation documentation for Cluster
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticmapreduce-cluster.html>`_
    """

    RESOURCE_TYPE = "AWS::EMR::Cluster"

    Properties: ClusterProperties = attrib(
        factory=ClusterProperties,
        converter=create_object_converter(ClusterProperties),
    )


@attrs(**ATTRSCONFIG)
class InstanceFleetConfigProperties(ResourceProperties):
    ClusterId = attrib(default=None)
    InstanceFleetType = attrib(default=None)
    InstanceTypeConfigs = attrib(default=None)
    LaunchSpecifications = attrib(default=None)
    Name = attrib(default=None)
    TargetOnDemandCapacity = attrib(default=None)
    TargetSpotCapacity = attrib(default=None)


@attrs(**ATTRSCONFIG)
class InstanceFleetConfig(Resource):
    """A Instance Fleet Config for EMR.

    See Also:
        `AWS Cloud Formation documentation for InstanceFleetConfig
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticmapreduce-instancefleetconfig.html>`_
    """

    RESOURCE_TYPE = "AWS::EMR::InstanceFleetConfig"

    Properties: InstanceFleetConfigProperties = attrib(
        factory=InstanceFleetConfigProperties,
        converter=create_object_converter(InstanceFleetConfigProperties),
    )


@attrs(**ATTRSCONFIG)
class InstanceGroupConfigProperties(ResourceProperties):
    AutoScalingPolicy = attrib(default=None)
    BidPrice = attrib(default=None)
    Configurations = attrib(default=None)
    EbsConfiguration = attrib(default=None)
    InstanceCount = attrib(default=None)
    InstanceRole = attrib(default=None)
    InstanceType = attrib(default=None)
    JobFlowId = attrib(default=None)
    Market = attrib(default=None)
    Name = attrib(default=None)


@attrs(**ATTRSCONFIG)
class InstanceGroupConfig(Resource):
    """A Instance Group Config for EMR.

    See Also:
        `AWS Cloud Formation documentation for InstanceGroupConfig
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-instancegroupconfig.html>`_
    """

    RESOURCE_TYPE = "AWS::EMR::InstanceGroupConfig"

    Properties: InstanceGroupConfigProperties = attrib(
        factory=InstanceGroupConfigProperties,
        converter=create_object_converter(InstanceGroupConfigProperties),
    )


@attrs(**ATTRSCONFIG)
class SecurityConfigurationProperties(ResourceProperties):
    Name = attrib(default=None)
    SecurityConfiguration = attrib(default=None)


@attrs(**ATTRSCONFIG)
class SecurityConfiguration(Resource):
    """A Security Configuration for EMR.

    See Also:
        `AWS Cloud Formation documentation for SecurityConfiguration
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-securityconfiguration.html>`_
    """

    RESOURCE_TYPE = "AWS::EMR::SecurityConfiguration"

    Properties: SecurityConfigurationProperties = attrib(
        factory=SecurityConfigurationProperties,
        converter=create_object_converter(SecurityConfigurationProperties),
    )


@attrs(**ATTRSCONFIG)
class StepProperties(ResourceProperties):
    ActionOnFailure = attrib(default=None)
    HadoopJarStep = attrib(default=None)
    JobFlowId = attrib(default=None)
    Name = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Step(Resource):
    """A Step for EMR.

    See Also:
        `AWS Cloud Formation documentation for Step
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-step.html>`_
    """

    RESOURCE_TYPE = "AWS::EMR::Step"

    Properties: StepProperties = attrib(
        factory=StepProperties,
        converter=create_object_converter(StepProperties),
    )



"""General-use classes to interact with the SageMaker service through CloudFormation.

See Also:
    `AWS developer guide for SageMaker
    <https://docs.aws.amazon.com/sagemaker/latest/dg/whatis.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import sagemaker as _raw

# noinspection PyUnresolvedReferences
from .._raw.sagemaker import *



"""General-use classes to interact with the S3 service through CloudFormation.

See Also:
    `AWS developer guide for S3
    <https://docs.aws.amazon.com/AmazonS3/latest/dev/Welcome.html>`_
"""

from .._raw import s3 as _raw
# noinspection PyUnresolvedReferences
from .._raw.s3 import *


class Bucket(_raw.Bucket):
    __slots__ = []

    #: Key ID for the AWS-supplied KMS Master Key used for server-side
    #: encryption if a customer-managed key is not explicitly specified.
    SSE_KMS_DEFAULT_MASTER_KEY = "aws/s3"

    def set_encryption(self, kms_keyid=SSE_KMS_DEFAULT_MASTER_KEY, s3_managed: bool = False):
        """Set the default server-side encryption approach for this bucket.

        This convenience function can be used to set any one of the mutually
        exclusive `AWS server-side encryption schemes
        <https://docs.aws.amazon.com/AmazonS3/latest/dev/serv-side-encryption.html>`_
        With default parameters, it will set KMS-managed encryption with the
        default AWS S3 KMS key.

        Parameters:
            kms_keyid: Use KMS-managed encryption with this custom KMS key.
            s3_managed: Use S3-managed encryption.
        """
        # TODO also implement is_s3_encrypted, is_kms_encrypted, get_kms_key

        # Checks
        if s3_managed and kms_keyid != self.SSE_KMS_DEFAULT_MASTER_KEY:
            raise ValueError("Unable to set both S3-managed and KMS-managed bucket encryption")

        # Get the top-level encryption config object
        config = self.Properties["BucketEncryption"]
        if not config:
            self.Properties["BucketEncryption"] = {}
            config = self.Properties["BucketEncryption"] = {}

        # Create a fresh single-value list of server-side encryption rules
        if s3_managed:
            rule = {"SSEAlgorithm": "AES256"}
        else:
            rule = {
                "KMSMasterKeyID": kms_keyid,
                "SSEAlgorithm": "aws:kms",
            }
        config["ServerSideEncryptionConfiguration"] = [{"ServerSideEncryptionByDefault": rule}]

    #: List of Status values for Bucket versioning, and whether they mean
    #: versioning is enabled
    _VERSIONING_STATUSES = {
        "Enabled": True,
        "Suspended": False,  # Default
    }

    @property
    def versioning(self) -> bool:
        """Whether versioning is enabled for this bucket.

        This sets the nested CloudFormation property "VersioningConfiguration.Status".

        Raises:
            ValueError: If the existing property does not have a valid value.

        See Also:
            `VersioningConfiguration documentation
            <https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-versioningconfig.html>`_
        """
        config = self.Properties["VersioningConfiguration"]
        if not config:
            return False
        status = config["Status"]

        try:
            return self._VERSIONING_STATUSES[status]
        except KeyError:
            raise ValueError(f"Unknown VersioningConfiguration status: {status}")

    @versioning.setter
    def versioning(self, value: bool):
        config = self.Properties["VersioningConfiguration"]
        if not config:
            self.Properties["VersioningConfiguration"] = {}
            config = self.Properties["VersioningConfiguration"]
        config["Status"] = "Enabled" if value else "Suspended"



"""General-use classes to interact with the IAM service through CloudFormation.

See Also:
    `AWS developer guide for IAM
    <https://docs.aws.amazon.com/IAM/latest/UserGuide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import iam as _raw

# noinspection PyUnresolvedReferences
from .._raw.iam import *



"""General-use classes to interact with the ElasticBeanstalk service through CloudFormation.

See Also:
    `AWS developer guide for ElasticBeanstalk
    <https://docs.aws.amazon.com/elasticbeanstalk/latest/dg/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import elasticbeanstalk as _raw

# noinspection PyUnresolvedReferences
from .._raw.elasticbeanstalk import *



"""General-use classes to interact with the IoTAnalytics service through CloudFormation.

See Also:
    `AWS developer guide for IoTAnalytics
    <https://docs.aws.amazon.com/appsync/latest/devguide/welcome.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import iotanalytics as _raw

# noinspection PyUnresolvedReferences
from .._raw.iotanalytics import *



"""Raw representations of every data type in the AWS ElastiCache service.

See Also:
    `AWS developer guide for ElastiCache
    <https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from typing import Any
from typing import Dict

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "CacheCluster",
    "CacheClusterProperties",
    "ParameterGroup",
    "ParameterGroupProperties",
    "ReplicationGroup",
    "ReplicationGroupProperties",
    "SecurityGroup",
    "SecurityGroupProperties",
    "SecurityGroupIngress",
    "SecurityGroupIngressProperties",
    "SubnetGroup",
    "SubnetGroupProperties",
]


@attrs(**ATTRSCONFIG)
class CacheClusterProperties(ResourceProperties):
    AutoMinorVersionUpgrade = attrib(default=None)
    AZMode = attrib(default=None)
    CacheNodeType = attrib(default=None)
    CacheParameterGroupName = attrib(default=None)
    CacheSecurityGroupNames = attrib(default=None)
    CacheSubnetGroupName = attrib(default=None)
    ClusterName = attrib(default=None)
    Engine = attrib(default=None)
    EngineVersion = attrib(default=None)
    NotificationTopicArn = attrib(default=None)
    NumCacheNodes = attrib(default=None)
    Port = attrib(default=None)
    PreferredAvailabilityZone = attrib(default=None)
    PreferredAvailabilityZones = attrib(default=None)
    PreferredMaintenanceWindow = attrib(default=None)
    SnapshotArns = attrib(default=None)
    SnapshotName = attrib(default=None)
    SnapshotRetentionLimit = attrib(default=None)
    SnapshotWindow = attrib(default=None)
    Tags = attrib(default=None)
    VpcSecurityGroupIds = attrib(default=None)


@attrs(**ATTRSCONFIG)
class CacheCluster(Resource):
    """A Cache Cluster for ElastiCache.

    See Also:
        `AWS Cloud Formation documentation for CacheCluster
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html>`_
    """

    RESOURCE_TYPE = "AWS::ElastiCache::CacheCluster"

    Properties: CacheClusterProperties = attrib(
        factory=CacheClusterProperties,
        converter=create_object_converter(CacheClusterProperties),
    )


@attrs(**ATTRSCONFIG)
class ParameterGroupProperties(ResourceProperties):
    CacheParameterGroupFamily = attrib(default=None)
    Description = attrib(default=None)
    Properties = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ParameterGroup(Resource):
    """A Parameter Group for ElastiCache.

    See Also:
        `AWS Cloud Formation documentation for ParameterGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-parameter-group.html>`_
    """

    RESOURCE_TYPE = "AWS::ElastiCache::ParameterGroup"

    Properties: ParameterGroupProperties = attrib(
        factory=ParameterGroupProperties,
        converter=create_object_converter(ParameterGroupProperties),
    )


@attrs(**ATTRSCONFIG)
class ReplicationGroupProperties(ResourceProperties):
    AtRestEncryptionEnabled = attrib(default=None)
    AuthToken = attrib(default=None)
    AutomaticFailoverEnabled = attrib(default=None)
    AutoMinorVersionUpgrade = attrib(default=None)
    CacheNodeType = attrib(default=None)
    CacheParameterGroupName = attrib(default=None)
    CacheSecurityGroupNames = attrib(default=None)
    CacheSubnetGroupName = attrib(default=None)
    Engine = attrib(default=None)
    EngineVersion = attrib(default=None)
    NodeGroupConfiguration = attrib(default=None)
    NotificationTopicArn = attrib(default=None)
    NumCacheClusters = attrib(default=None)
    NumNodeGroups = attrib(default=None)
    Port = attrib(default=None)
    PreferredCacheClusterAZs = attrib(default=None)
    PreferredMaintenanceWindow = attrib(default=None)
    PrimaryClusterId = attrib(default=None)
    ReplicasPerNodeGroup = attrib(default=None)
    ReplicationGroupDescription = attrib(default=None)
    ReplicationGroupId = attrib(default=None)
    SecurityGroupIds = attrib(default=None)
    SnapshotArns = attrib(default=None)
    SnapshotName = attrib(default=None)
    SnapshotRetentionLimit = attrib(default=None)
    SnapshottingClusterId = attrib(default=None)
    SnapshotWindow = attrib(default=None)
    Tags = attrib(default=None)
    TransitEncryptionEnabled = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ReplicationGroup(Resource):
    """A Replication Group for ElastiCache.

    See Also:
        `AWS Cloud Formation documentation for ReplicationGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html>`_
    """

    RESOURCE_TYPE = "AWS::ElastiCache::ReplicationGroup"

    Properties: ReplicationGroupProperties = attrib(
        factory=ReplicationGroupProperties,
        converter=create_object_converter(ReplicationGroupProperties),
    )

    # NB: UpdatePolicy may be set for ReplicationGroup
    # (unlike most Resource types)
    UpdatePolicy: Dict[str, Any] = attrib(factory=dict)


@attrs(**ATTRSCONFIG)
class SecurityGroupProperties(ResourceProperties):
    Description = attrib(default=None)


@attrs(**ATTRSCONFIG)
class SecurityGroup(Resource):
    """A Security Group for ElastiCache.

    See Also:
        `AWS Cloud Formation documentation for SecurityGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-security-group.html>`_
    """

    RESOURCE_TYPE = "AWS::ElastiCache::SecurityGroup"

    Properties: SecurityGroupProperties = attrib(
        factory=SecurityGroupProperties,
        converter=create_object_converter(SecurityGroupProperties),
    )


@attrs(**ATTRSCONFIG)
class SecurityGroupIngressProperties(ResourceProperties):
    CacheSecurityGroupName = attrib(default=None)
    EC2SecurityGroupName = attrib(default=None)
    EC2SecurityGroupOwnerId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class SecurityGroupIngress(Resource):
    """A Security Group Ingress for ElastiCache.

    See Also:
        `AWS Cloud Formation documentation for SecurityGroupIngress
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-security-group-ingress.html>`_
    """

    RESOURCE_TYPE = "AWS::ElastiCache::SecurityGroupIngress"

    Properties: SecurityGroupIngressProperties = attrib(
        factory=SecurityGroupIngressProperties,
        converter=create_object_converter(SecurityGroupIngressProperties),
    )


@attrs(**ATTRSCONFIG)
class SubnetGroupProperties(ResourceProperties):
    CacheSubnetGroupName = attrib(default=None)
    Description = attrib(default=None)
    SubnetIds = attrib(default=None)


@attrs(**ATTRSCONFIG)
class SubnetGroup(Resource):
    """A Subnet Group for ElastiCache.

    See Also:
        `AWS Cloud Formation documentation for SubnetGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-subnetgroup.html>`_
    """

    RESOURCE_TYPE = "AWS::ElastiCache::SubnetGroup"

    Properties: SubnetGroupProperties = attrib(
        factory=SubnetGroupProperties,
        converter=create_object_converter(SubnetGroupProperties),
    )



"""Raw representations of every data type in the AWS SES service.

See Also:
    `AWS developer guide for SES
    <https://docs.aws.amazon.com/servicecatalog/latest/dg/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "ConfigurationSet",
    "ConfigurationSetProperties",
    "ConfigurationSetEventDestination",
    "ConfigurationSetEventDestinationProperties",
    "ReceiptFilter",
    "ReceiptFilterProperties",
    "ReceiptRule",
    "ReceiptRuleProperties",
    "ReceiptRuleSet",
    "ReceiptRuleSetProperties",
    "Template",
    "TemplateProperties",
]


@attrs(**ATTRSCONFIG)
class ConfigurationSetProperties(ResourceProperties):
    Name = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ConfigurationSet(Resource):
    """A Configuration Set for SES.

    See Also:
        `AWS Cloud Formation documentation for ConfigurationSet
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ses-configurationset.html>`_
    """

    RESOURCE_TYPE = "AWS::SES::ConfigurationSet"

    Properties: ConfigurationSetProperties = attrib(
        factory=ConfigurationSetProperties,
        converter=create_object_converter(ConfigurationSetProperties),
    )


@attrs(**ATTRSCONFIG)
class ConfigurationSetEventDestinationProperties(ResourceProperties):
    ConfigurationSetName = attrib(default=None)
    EventDestination = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ConfigurationSetEventDestination(Resource):
    """A Configuration Set Event Destination for SES.

    See Also:
        `AWS Cloud Formation documentation for ConfigurationSetEventDestination
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ses-configurationseteventdestination.html>`_
    """

    RESOURCE_TYPE = "AWS::SES::ConfigurationSetEventDestination"

    Properties: ConfigurationSetEventDestinationProperties = attrib(
        factory=ConfigurationSetEventDestinationProperties,
        converter=create_object_converter(ConfigurationSetEventDestinationProperties),
    )


@attrs(**ATTRSCONFIG)
class ReceiptFilterProperties(ResourceProperties):
    Filter = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ReceiptFilter(Resource):
    """A Receipt Filter for SES.

    See Also:
        `AWS Cloud Formation documentation for ReceiptFilter
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ses-receiptfilter.html>`_
    """

    RESOURCE_TYPE = "AWS::SES::ReceiptFilter"

    Properties: ReceiptFilterProperties = attrib(
        factory=ReceiptFilterProperties,
        converter=create_object_converter(ReceiptFilterProperties),
    )


@attrs(**ATTRSCONFIG)
class ReceiptRuleProperties(ResourceProperties):
    After = attrib(default=None)
    Rule = attrib(default=None)
    RuleSetName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ReceiptRule(Resource):
    """A Receipt Rule for SES.

    See Also:
        `AWS Cloud Formation documentation for ReceiptRule
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ses-receiptrule.html>`_
    """

    RESOURCE_TYPE = "AWS::SES::ReceiptRule"

    Properties: ReceiptRuleProperties = attrib(
        factory=ReceiptRuleProperties,
        converter=create_object_converter(ReceiptRuleProperties),
    )


@attrs(**ATTRSCONFIG)
class ReceiptRuleSetProperties(ResourceProperties):
    RuleSetName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ReceiptRuleSet(Resource):
    """A Receipt Rule Set for SES.

    See Also:
        `AWS Cloud Formation documentation for ReceiptRuleSet
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ses-receiptruleset.html>`_
    """

    RESOURCE_TYPE = "AWS::SES::ReceiptRuleSet"

    Properties: ReceiptRuleSetProperties = attrib(
        factory=ReceiptRuleSetProperties,
        converter=create_object_converter(ReceiptRuleSetProperties),
    )


@attrs(**ATTRSCONFIG)
class TemplateProperties(ResourceProperties):
    Template = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Template(Resource):
    """A Template for SES.

    See Also:
        `AWS Cloud Formation documentation for Template
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ses-template.html>`_
    """

    RESOURCE_TYPE = "AWS::SES::Template"

    Properties: TemplateProperties = attrib(
        factory=TemplateProperties,
        converter=create_object_converter(TemplateProperties),
    )
